@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.29 2008/06/07 07:27:28 nicm Exp $ */
+/* $Id: client.c,v 1.33 2008/06/23 16:58:49 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,7 +24,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <paths.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
@@ -45,9 +44,8 @@ client_init(const char *path, struct client_ctx *cctx, int start_server)
 	int				mode;
 	u_int				retries;
 	struct buffer		       *b;
-	pid_t				pid;
+	char			       *name;
 
-	pid = 0;
 	retries = 0;
 retry:
 	if (stat(path, &sb) != 0) {
@@ -97,8 +95,12 @@ retry:
 			fatal("ioctl(TIOCGWINSZ)");
 		data.sx = ws.ws_col;
 		data.sy = ws.ws_row;
-		if (ttyname_r(STDIN_FILENO, data.tty, sizeof data.tty) != 0)
-			fatal("ttyname_r failed");
+		*data.tty = '\0';
+
+		if ((name = ttyname(STDIN_FILENO)) == NULL)
+			fatal("ttyname failed");
+		if (strlcpy(data.tty, name, sizeof data.tty) >= sizeof data.tty)
+			fatalx("ttyname failed");
 
 		b = buffer_create(BUFSIZ);
 		cmd_send_string(b, getenv("TERM"));
@@ -123,7 +125,7 @@ client_main(struct client_ctx *cctx)
 {
 	struct pollfd	 pfd;
 	char		*error;
-	int		 timeout;
+	int		 xtimeout; /* Yay for ncurses namespace! */
 
 	siginit();
 
@@ -133,7 +135,7 @@ client_main(struct client_ctx *cctx)
 #endif
 
 	error = NULL;
-	timeout = INFTIM;
+	xtimeout = INFTIM;
 	while (!sigterm) {
 		if (sigwinch)
 			client_handle_winch(cctx);
@@ -143,11 +145,11 @@ client_main(struct client_ctx *cctx)
 			goto out;
 		case 0:
 			/* May be more in buffer, don't let poll block. */
-			timeout = 0;
+			xtimeout = 0;
 			break;
 		default:
 			/* Out of data, poll may block. */
-			timeout = INFTIM;
+			xtimeout = INFTIM;
 			break;
 		}
 
@@ -156,7 +158,7 @@ client_main(struct client_ctx *cctx)
 		if (BUFFER_USED(cctx->srv_out) > 0)
 			pfd.events |= POLLOUT;
 
-		if (poll(&pfd, 1, timeout) == -1) {
+		if (poll(&pfd, 1, xtimeout) == -1) {
 			if (errno == EAGAIN || errno == EINTR)
 				continue;
 			fatal("poll failed");
