@@ -1,4 +1,4 @@
-/* $Id: cmd-bind-key.c,v 1.8 2007/12/06 09:46:21 nicm Exp $ */
+/* $Id: cmd-bind-key.c,v 1.15 2008/06/05 21:54:47 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,11 +26,12 @@
  * Bind a key to a command, this recurses through cmd_*.
  */
 
-int	cmd_bind_key_parse(void **, int, char **, char **);
-void	cmd_bind_key_exec(void *, struct cmd_ctx *);
-void	cmd_bind_key_send(void *, struct buffer *);
-void	cmd_bind_key_recv(void **, struct buffer *);
-void	cmd_bind_key_free(void *);
+int	cmd_bind_key_parse(struct cmd *, int, char **, char **);
+void	cmd_bind_key_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_bind_key_send(struct cmd *, struct buffer *);
+void	cmd_bind_key_recv(struct cmd *, struct buffer *);
+void	cmd_bind_key_free(struct cmd *);
+void	cmd_bind_key_print(struct cmd *, char *, size_t);
 
 struct cmd_bind_key_data {
 	int		 key;
@@ -38,22 +39,25 @@ struct cmd_bind_key_data {
 };
 
 const struct cmd_entry cmd_bind_key_entry = {
-	"bind-key", "bind", "key command [arguments]",
-	CMD_NOCLIENT|CMD_NOSESSION,
+	"bind-key", "bind",
+	"key command [arguments]",
+	0,
+	NULL,
 	cmd_bind_key_parse,
 	cmd_bind_key_exec,
 	cmd_bind_key_send,
 	cmd_bind_key_recv,
-	cmd_bind_key_free
+	cmd_bind_key_free,
+	cmd_bind_key_print
 };
 
 int
-cmd_bind_key_parse(void **ptr, int argc, char **argv, char **cause)
+cmd_bind_key_parse(struct cmd *self, int argc, char **argv, char **cause)
 {
 	struct cmd_bind_key_data	*data;
 	int				 opt;
 
-	*ptr = data = xmalloc(sizeof *data);
+	self->data = data = xmalloc(sizeof *data);
 	data->cmd = NULL;
 
 	while ((opt = getopt(argc, argv, "")) != EOF) {
@@ -80,18 +84,17 @@ cmd_bind_key_parse(void **ptr, int argc, char **argv, char **cause)
 	return (0);
 
 usage:
-	usage(cause, "%s %s",
-	    cmd_bind_key_entry.name, cmd_bind_key_entry.usage);
+	xasprintf(cause, "usage: %s %s", self->entry->name, self->entry->usage);
 
 error:
-	cmd_bind_key_free(data);
+	self->entry->free(self);
 	return (-1);
 }
 
 void
-cmd_bind_key_exec(void *ptr, unused struct cmd_ctx *ctx)
+cmd_bind_key_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_bind_key_data	*data = ptr;
+	struct cmd_bind_key_data	*data = self->data;
 
 	if (data == NULL)
 		return;
@@ -104,30 +107,49 @@ cmd_bind_key_exec(void *ptr, unused struct cmd_ctx *ctx)
 }
 
 void
-cmd_bind_key_send(void *ptr, struct buffer *b)
+cmd_bind_key_send(struct cmd *self, struct buffer *b)
 {
-	struct cmd_bind_key_data	*data = ptr;
+	struct cmd_bind_key_data	*data = self->data;
 
 	buffer_write(b, data, sizeof *data);
 	cmd_send(data->cmd, b);
 }
 
 void
-cmd_bind_key_recv(void **ptr, struct buffer *b)
+cmd_bind_key_recv(struct cmd *self, struct buffer *b)
 {
 	struct cmd_bind_key_data	*data;
 
-	*ptr = data = xmalloc(sizeof *data);
+	self->data = data = xmalloc(sizeof *data);
 	buffer_read(b, data, sizeof *data);
 	data->cmd = cmd_recv(b);
 }
 
 void
-cmd_bind_key_free(void *ptr)
+cmd_bind_key_free(struct cmd *self)
 {
-	struct cmd_bind_key_data	*data = ptr;
+	struct cmd_bind_key_data	*data = self->data;
 
 	if (data->cmd != NULL)
 		cmd_free(data->cmd);
 	xfree(data);
 }
+
+void
+cmd_bind_key_print(struct cmd *self, char *buf, size_t len)
+{
+	struct cmd_bind_key_data	*data = self->data;
+	size_t				 off = 0;
+
+	off += xsnprintf(buf, len, "%s", self->entry->name);
+	if (data == NULL)
+		return;
+	if (off < len) {
+		off += xsnprintf(buf + off,
+		    len - off, " %s", key_string_lookup_key(data->key));
+	}
+	if (off < len && data->cmd != NULL) {
+		off += xsnprintf(buf + off, len - off, " ");
+		data->cmd->entry->print(data->cmd, buf + off, len - off);
+	}
+}
