@@ -1,4 +1,4 @@
-/* $Id: cmd-list-keys.c,v 1.5 2007/11/16 21:12:31 nicm Exp $ */
+/* $Id: cmd-list-keys.c,v 1.10 2008/06/05 21:25:00 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,39 +18,45 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-#include <string.h>
-
 #include "tmux.h"
 
 /*
  * List key bindings.
  */
 
-void	cmd_list_keys_exec(void *, struct cmd_ctx *);
+void	cmd_list_keys_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_list_keys_entry = {
-	"list-keys", "lsk", "",
-	CMD_NOCLIENT|CMD_NOSESSION,
+	"list-keys", "lsk",
+	"",
+	0,
+	NULL,
 	NULL,
 	cmd_list_keys_exec,
 	NULL,
 	NULL,
+	NULL,
 	NULL
 };
 
 void
-cmd_list_keys_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_list_keys_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct binding	*bd;
 	const char	*key;
+	char		 s[BUFSIZ];
 	u_int		 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&key_bindings); i++) {
 		bd = ARRAY_ITEM(&key_bindings, i);
 		if ((key = key_string_lookup_key(bd->key)) == NULL)
 			continue;
-		ctx->print(ctx, "%11s: %s", key, bd->cmd->entry->name);
+		if (bd->cmd->entry->print == NULL) {
+			ctx->print(ctx, "%11s: %s", key, bd->cmd->entry->name);
+			continue;
+		}
+		bd->cmd->entry->print(bd->cmd, s, sizeof s);
+		ctx->print(ctx, "%11s: %s", key, s);
 	}
 
 	if (ctx->cmdclient != NULL)
