@@ -1,4 +1,4 @@
-/* $Id: cmd-new-session.c,v 1.27 2008/06/05 21:25:00 nicm Exp $ */
+/* $Id: cmd-new-session.c,v 1.30 2008/06/21 10:19:36 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -43,7 +43,7 @@ struct cmd_new_session_data {
 
 const struct cmd_entry cmd_new_session_entry = {
 	"new-session", "new",
-	"[-d] [-s session-name] [-n window-name] [command]",
+	"[-d] [-n window-name] [-s session-name] [command]",
 	CMD_STARTSERVER|CMD_CANTNEST,
 	cmd_new_session_init,
 	cmd_new_session_parse,
@@ -116,9 +116,9 @@ cmd_new_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct client			*c = ctx->cmdclient;
 	struct session			*s;
 	char				*cmd, *cause;
-	u_int				 sx, sy, slines;
+	u_int				 sx, sy;
 
-	if (ctx->flags & CMD_KEY)
+	if (ctx->curclient != NULL)
 		return;
 
 	if (!data->flag_detached) {
@@ -148,10 +148,12 @@ cmd_new_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 		sy = c->sy;
 	}
 
-	slines = options_get_number(&global_options, "status-lines");
-	if (sy < slines)
-		sy = slines + 1;
-	sy -= slines;
+	if (options_get_number(&global_options, "status")) {
+		if (sy == 0)
+			sy = 1;
+		else
+			sy--;
+	}
 
 	if (!data->flag_detached && tty_open(&c->tty, &cause) != 0) {
 		ctx->error(ctx, "%s", cause);
