@@ -1,4 +1,4 @@
-/* $Id: cmd-set-window-option.c,v 1.8 2008/06/16 17:35:40 nicm Exp $ */
+/* $Id: cmd-set-window-option.c,v 1.11 2008/06/29 07:04:30 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -101,7 +101,7 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct winlink				*wl;
 	struct session				*s;
 	const char				*errstr;
-	int				 	 number, bool;
+	int				 	 number, flag;
 	u_int					 i;
 
 	if (data == NULL)
@@ -120,28 +120,28 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 	if (data->value != NULL) {
 		number = strtonum(data->value, 0, INT_MAX, &errstr);
 
-		bool = -1;
+		flag = -1;
 		if (number == 1 || strcasecmp(data->value, "on") == 0 ||
 		    strcasecmp(data->value, "yes") == 0)
-			bool = 1;
+			flag = 1;
 		else if (number == 0 || strcasecmp(data->value, "off") == 0 ||
 		    strcasecmp(data->value, "no") == 0)
-			bool = 0;
+			flag = 0;
 	} else
-		bool = -2;
+		flag = -2;
 
 	if (strcmp(data->option, "monitor-activity") == 0) {
-		if (bool == -1) {
+		if (flag == -1) {
 			ctx->error(ctx, "bad value: %s", data->value);
 			return;
 		}
 
-		if (bool == -2)
+		if (flag == -2)
 			wl->window->flags ^= WINDOW_MONITOR;
 		else {
-			if (bool)
+			if (flag)
 				wl->window->flags |= WINDOW_MONITOR;
-			else 
+			else
 				wl->window->flags &= ~WINDOW_MONITOR;
 		}
 
@@ -154,22 +154,22 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 
 		for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
-			s = ARRAY_ITEM(&sessions, i); 
+			s = ARRAY_ITEM(&sessions, i);
 			if (s != NULL)
 				session_alert_cancel(s, wl);
 		}
 	} else if (strcmp(data->option, "aggressive-resize") == 0) {
-		if (bool == -1) {
+		if (flag == -1) {
 			ctx->error(ctx, "bad value: %s", data->value);
 			return;
 		}
 
-		if (bool == -2)
+		if (flag == -2)
 			wl->window->flags ^= WINDOW_AGGRESSIVE;
 		else {
-			if (bool)
+			if (flag)
 				wl->window->flags |= WINDOW_AGGRESSIVE;
-			else 
+			else
 				wl->window->flags &= ~WINDOW_AGGRESSIVE;
 		}
 
@@ -218,6 +218,20 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 		    s->name, wl->idx, number);
 
 		recalculate_sizes();
+	} else if (strcmp(data->option, "remain-on-exit") == 0) {
+		if (flag == -1) {
+			ctx->error(ctx, "bad value: %s", data->value);
+			return;
+		}
+
+		if (flag == -2)
+			wl->window->flags ^= WINDOW_ZOMBIFY;
+		else {
+			if (flag)
+				wl->window->flags |= WINDOW_ZOMBIFY;
+			else
+				wl->window->flags &= ~WINDOW_ZOMBIFY;
+		}
 	} else {
 		ctx->error(ctx, "unknown option: %s", data->option);
 		return;
@@ -278,5 +292,5 @@ cmd_set_window_option_print(struct cmd *self, char *buf, size_t len)
 	if (off < len && data->option != NULL)
 		off += xsnprintf(buf + off, len - off, " %s", data->option);
 	if (off < len && data->value != NULL)
-		off += xsnprintf(buf + off, len - off, " %s", data->value);	
+		off += xsnprintf(buf + off, len - off, " %s", data->value);
 }
