@@ -1,4 +1,4 @@
-/* $Id: cmd-show-options.c,v 1.2 2008/06/16 06:33:50 nicm Exp $ */
+/* $Id: cmd-show-options.c,v 1.5 2008/06/23 22:12:29 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -98,7 +98,10 @@ cmd_show_options_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_show_options_data	*data = self->data;
 	struct session			*s;
 	struct options			*oo;
-	struct options_entry		*o;
+	const struct set_option_entry   *entry;
+	u_int				 i;
+	char				*vs;
+	long long			 vn;
 
 	if (data == NULL)
 		return;
@@ -109,23 +112,42 @@ cmd_show_options_exec(struct cmd *self, struct cmd_ctx *ctx)
 	else
 		oo = &s->options;
 
-	SPLAY_FOREACH(o, options_tree, &oo->tree) {
-		switch (o->type) {
-		case OPTIONS_STRING:
-			ctx->print(
-			    ctx, "%s \"%s\"", o->name, o->value.string);
+	for (i = 0; i < NSETOPTION; i++) {
+		entry = &set_option_table[i];
+
+		if (options_find1(oo, entry->name) == NULL)
+			continue;
+
+		switch (entry->type) {
+		case SET_OPTION_STRING:
+			vs = options_get_string(oo, entry->name);
+			ctx->print(ctx, "%s \"%s\"", entry->name, vs);
 			break;
-		case OPTIONS_NUMBER:
-			ctx->print(ctx, "%s %lld", o->name, o->value.number);
+		case SET_OPTION_NUMBER:
+			vn = options_get_number(oo, entry->name);
+			ctx->print(ctx, "%s %lld", entry->name, vn);
 			break;
-		case OPTIONS_KEY:
-			ctx->print(ctx, "%s %s", o->name, 
-			    key_string_lookup_key(o->value.key));
+		case SET_OPTION_KEY:
+			vn = options_get_number(oo, entry->name);
+ 			ctx->print(ctx, "%s %s",
+			    entry->name, key_string_lookup_key(vn));
 			break;
-		case OPTIONS_COLOURS:
-			ctx->print(ctx, "%s fg=%s, bg=%s", o->name,
-			    screen_colourstring(o->value.colours >> 4),
-			    screen_colourstring(o->value.colours & 0x0f));
+		case SET_OPTION_COLOUR:
+			vn = options_get_number(oo, entry->name);
+ 			ctx->print(ctx, "%s %s",
+			    entry->name, screen_colourstring(vn));
+			break;
+		case SET_OPTION_FLAG:
+			vn = options_get_number(oo, entry->name);
+			if (vn)
+				ctx->print(ctx, "%s on", entry->name);
+			else
+				ctx->print(ctx, "%s off", entry->name);
+			break;
+		case SET_OPTION_CHOICE:
+			vn = options_get_number(oo, entry->name);
+			ctx->print(ctx, "%s %s",
+			    entry->name, entry->choices[vn]);
 			break;
 		}
 	}
