@@ -1,4 +1,4 @@
-/* $Id: cmd-switch-client.c,v 1.9 2008/06/05 21:25:00 nicm Exp $ */
+/* $Id: cmd-switch-client.c,v 1.11 2008/06/23 22:26:52 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -42,7 +42,7 @@ struct cmd_switch_client_data {
 
 const struct cmd_entry cmd_switch_client_entry = {
 	"switch-client", "switchc",
-	"[-c client-tty] [-t target-session]",
+	"[-c target-client] [-t target-session]",
 	0,
 	NULL,
 	cmd_switch_client_parse,
@@ -103,7 +103,7 @@ cmd_switch_client_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return;
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
 		return;
-	
+
 	c->session = s;
 
 	recalculate_sizes();
