@@ -1,4 +1,4 @@
-/* $Id: cmd.c,v 1.48 2008/06/16 22:03:27 nicm Exp $ */
+/* $Id: cmd.c,v 1.60 2008/06/29 07:04:30 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,7 +29,9 @@
 const struct cmd_entry *cmd_table[] = {
 	&cmd_attach_session_entry,
 	&cmd_bind_key_entry,
+	&cmd_command_prompt_entry,
 	&cmd_copy_mode_entry,
+	&cmd_delete_buffer_entry,
 	&cmd_detach_client_entry,
 	&cmd_has_session_entry,
 	&cmd_kill_server_entry,
@@ -37,10 +39,13 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_kill_window_entry,
 	&cmd_last_window_entry,
 	&cmd_link_window_entry,
+	&cmd_list_buffers_entry,
 	&cmd_list_clients_entry,
+	&cmd_list_commands_entry,
 	&cmd_list_keys_entry,
 	&cmd_list_sessions_entry,
 	&cmd_list_windows_entry,
+	&cmd_move_window_entry,
 	&cmd_new_session_entry,
 	&cmd_new_window_entry,
 	&cmd_next_window_entry,
@@ -49,12 +54,16 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_refresh_client_entry,
 	&cmd_rename_session_entry,
 	&cmd_rename_window_entry,
+	&cmd_respawn_window_entry,
 	&cmd_scroll_mode_entry,
+	&cmd_select_prompt_entry,
 	&cmd_select_window_entry,
 	&cmd_send_keys_entry,
 	&cmd_send_prefix_entry,
+	&cmd_set_buffer_entry,
 	&cmd_set_option_entry,
 	&cmd_set_window_option_entry,
+	&cmd_show_buffer_entry,
 	&cmd_show_options_entry,
 	&cmd_show_window_options_entry,
 	&cmd_start_server_entry,
@@ -65,6 +74,57 @@ const struct cmd_entry *cmd_table[] = {
 	NULL
 };
 
+char *
+cmd_complete(const char *s)
+{
+	const struct cmd_entry 	      **entryp;
+	ARRAY_DECL(, const char *)	list;
+	char			       *prefix, *s2;
+	u_int			 	i;
+	size_t			 	j;
+
+	if (*s == '\0')
+		return (xstrdup(s));
+
+	/* First, build a list of all the possible matches. */
+	ARRAY_INIT(&list);
+	for (entryp = cmd_table; *entryp != NULL; entryp++) {
+		if (strncmp((*entryp)->name, s, strlen(s)) != 0)
+			continue;
+		ARRAY_ADD(&list, (*entryp)->name);
+	}
+	
+	/* If none, bail now with the original string. */
+	if (ARRAY_LENGTH(&list) == 0) {
+		ARRAY_FREE(&list);
+		return (xstrdup(s));
+	}
+	
+	/* If an exact match, return it, with a trailing space. */
+	if (ARRAY_LENGTH(&list) == 1) {
+		xasprintf(&s2, "%s ", ARRAY_FIRST(&list));
+		ARRAY_FREE(&list);
+		return (s2);
+	}
+
+	/* Now loop through the list and find the longest common prefix. */
+	prefix = xstrdup(ARRAY_FIRST(&list));
+	for (i = 1; i < ARRAY_LENGTH(&list); i++) {
+		s = ARRAY_ITEM(&list, i);
+
+		j = strlen(s);
+		if (j > strlen(prefix))
+			j = strlen(prefix);
+		for (; j > 0; j--) {
+			if (prefix[j - 1] != s[j - 1])
+				prefix[j - 1] = '\0';
+		}
+	}
+
+	ARRAY_FREE(&list);
+	return (prefix);
+}
+
 struct cmd *
 cmd_parse(int argc, char **argv, char **cause)
 {
@@ -262,10 +322,10 @@ cmd_current_session(struct cmd_ctx *ctx)
 		}
 		return (s);
 	}
-	
+
 	ts = NULL;
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
-		s = ARRAY_ITEM(&sessions, i); 
+		s = ARRAY_ITEM(&sessions, i);
 		if (s != NULL && (ts == NULL || timespeccmp(&s->ts, ts, >))) {
 			newest = ARRAY_ITEM(&sessions, i);
 			ts = &s->ts;
@@ -278,14 +338,16 @@ struct client *
 cmd_find_client(struct cmd_ctx *ctx, const char *arg)
 {
 	struct client	*c;
-	
-	if ((c = arg_parse_client(arg)) == NULL)
+
+	if (arg == NULL)
 		c = ctx->curclient;
-	if (c == NULL) {
-		if (arg != NULL)
-			ctx->error(ctx, "client not found: %s", arg);
-		else
-			ctx->error(ctx, "no client found");
+	else {
+		if ((c = arg_parse_client(arg)) == NULL) {
+			if (arg != NULL)
+				ctx->error(ctx, "client not found: %s", arg);
+			else
+				ctx->error(ctx, "no client found");
+		}
 	}
 	return (c);
 }
@@ -295,15 +357,15 @@ cmd_find_session(struct cmd_ctx *ctx, const char *arg)
 {
 	struct session	*s;
 
-	if ((s = arg_parse_session(arg)) == NULL)
-		s = ctx->cursession;
-	if (s == NULL)
+	if (arg == NULL)
 		s = cmd_current_session(ctx);
-	if (s == NULL) {
-		if (arg != NULL)
-			ctx->error(ctx, "session not found: %s", arg);
-		else
-			ctx->error(ctx, "no session found");
+	else {
+		if ((s = arg_parse_session(arg)) == NULL) {
+			if (arg != NULL)
+				ctx->error(ctx, "session not found: %s", arg);
+			else
+				ctx->error(ctx, "no session found");
+		}
 	}
 	return (s);
 }
