@@ -1,4 +1,4 @@
-/* $Id: key-bindings.c,v 1.31 2008/06/16 17:35:40 nicm Exp $ */
+/* $Id: key-bindings.c,v 1.37 2008/06/25 20:43:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,10 +26,6 @@
 
 struct bindings	key_bindings;
 
-void printflike2 key_bindings_error(struct cmd_ctx *, const char *, ...);
-void printflike2 key_bindings_print(struct cmd_ctx *, const char *, ...);
-void printflike2 key_bindings_info(struct cmd_ctx *, const char *, ...);
-
 void
 key_bindings_add(int key, struct cmd *cmd)
 {
@@ -80,22 +76,10 @@ key_bindings_init(void)
 		int			 key;
 		const struct cmd_entry	*entry;
 	} table[] = {
-		{ 'D', &cmd_detach_client_entry },
-		{ 'd', &cmd_detach_client_entry },
-		{ 'S', &cmd_list_sessions_entry },
-		{ 's', &cmd_list_sessions_entry },
-		{ 'W', &cmd_list_windows_entry },
-		{ 'w', &cmd_list_windows_entry },
-		{ '?', &cmd_list_keys_entry },
+		{ '#', &cmd_list_buffers_entry },
+		{ '&', &cmd_kill_window_entry },
+		{ '-', &cmd_delete_buffer_entry },
 		{ '/', &cmd_list_keys_entry },
-		{ 'C', &cmd_new_window_entry },
-		{ 'c', &cmd_new_window_entry },
-		{ 'N', &cmd_next_window_entry },
-		{ 'n', &cmd_next_window_entry },
-		{ 'P', &cmd_previous_window_entry },
-		{ 'p', &cmd_previous_window_entry },
-		{ 'L', &cmd_last_window_entry },
-		{ 'l', &cmd_last_window_entry },
 		{ '0', &cmd_select_window_entry },
 		{ '1', &cmd_select_window_entry },
 		{ '2', &cmd_select_window_entry },
@@ -106,12 +90,20 @@ key_bindings_init(void)
 		{ '7', &cmd_select_window_entry },
 		{ '8', &cmd_select_window_entry },
 		{ '9', &cmd_select_window_entry },
-		{ 'R', &cmd_refresh_client_entry },
-		{ 'r', &cmd_refresh_client_entry },
-		{ '&', &cmd_kill_window_entry },
+		{ ':', &cmd_command_prompt_entry },
 		{ '=', &cmd_scroll_mode_entry },
+		{ '?', &cmd_list_keys_entry },
 		{ '[', &cmd_copy_mode_entry },
 		{ ']', &cmd_paste_buffer_entry },
+		{ 'c', &cmd_new_window_entry },
+		{ 'd', &cmd_detach_client_entry },
+		{ 'l', &cmd_last_window_entry },
+		{ 'n', &cmd_next_window_entry },
+		{ 'p', &cmd_previous_window_entry },
+		{ 'r', &cmd_refresh_client_entry },
+		{ 's', &cmd_list_sessions_entry },
+		{ 'w', &cmd_list_windows_entry },
+		{ '\'', &cmd_select_prompt_entry },
 		{ META, &cmd_send_prefix_entry },
 	};
 	u_int		 i;
@@ -156,7 +148,7 @@ key_bindings_error(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_end(ap);
 
 	*msg = toupper((u_char) *msg);
-	server_write_message(ctx->curclient, "%s", msg);
+ 	server_set_client_message(ctx->curclient, msg);
 	xfree(msg);
 }
 
@@ -187,7 +179,7 @@ key_bindings_info(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_end(ap);
 
 	*msg = toupper((u_char) *msg);
-	server_write_message(ctx->curclient, "%s", msg);
+ 	server_set_client_message(ctx->curclient, msg);
 	xfree(msg);
 }
 
@@ -216,7 +208,6 @@ key_bindings_dispatch(int key, struct client *c)
 	ctx.info = key_bindings_info;
 
 	ctx.cmdclient = NULL;
-	ctx.flags = CMD_KEY;
 
 	cmd_exec(bd->cmd, &ctx);
 }
