@@ -1,4 +1,4 @@
-/* $Id: screen.c,v 1.61 2008/06/14 12:05:06 nicm Exp $ */
+/* $Id: screen.c,v 1.63 2008/06/29 07:04:30 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -45,7 +45,7 @@
  * ----------- array base + hsize + dy
  *
  * The screen_x/screen_y macros are used to convert a cell on the displayed
- * area to an absolute position in the arrays. 
+ * area to an absolute position in the arrays.
  *
  * Screen handling code is split into four files:
  *
@@ -54,7 +54,7 @@
  *	screen-display.c: Basic functions for manipulating the displayed
  *			  part of the screen. x,y coordinates passed to these
  *			  are relative to the display. These are largely
- *			  utility functions for screen-write.c. 
+ *			  utility functions for screen-write.c.
  *	screen-redraw.c: Functions for redrawing all or part of a screen to
  *			 one or more ttys. A context is filled via one of the
  *			 screen_redraw_start* variants which sets up (removes
@@ -156,6 +156,27 @@ screen_create(struct screen *s, u_int dx, u_int dy, u_int hlimit)
 	screen_clear_selection(s);
 }
 
+/* Reinitialise screen. */
+void
+screen_reset(struct screen *s)
+{
+	s->cx = 0;
+	s->cy = 0;
+
+	s->rupper = 0;
+	s->rlower = s->dy - 1;
+
+	s->attr = SCREEN_DEFATTR;
+	s->colr = SCREEN_DEFCOLR;
+
+	s->mode = MODE_CURSOR|MODE_KCURSOR|MODE_KKEYPAD;
+
+	screen_display_fill_area(s, 0, 0, 
+	    screen_size_x(s), screen_size_y(s), ' ', 0, 0x88);
+	
+	screen_clear_selection(s);	
+}
+
 /* Resize screen. */
 void
 screen_resize(struct screen *s, u_int sx, u_int sy)
