@@ -1,4 +1,4 @@
-/* $Id: server-fn.c,v 1.42 2008/06/16 17:35:40 nicm Exp $ */
+/* $Id: server-fn.c,v 1.49 2008/06/22 22:28:33 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,10 +19,74 @@
 #include <sys/types.h>
 
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "tmux.h"
 
+void
+server_set_client_message(struct client *c, const char *msg)
+{
+	struct timespec	ts;
+	int		delay;
+
+	delay = options_get_number(&c->session->options, "display-time");
+	ts.tv_sec = delay / 1000;
+	ts.tv_nsec = (delay % 1000) * 1000000L;
+
+	c->message_string = xstrdup(msg);
+	if (clock_gettime(CLOCK_REALTIME, &c->message_timer) != 0)
+		fatal("clock_gettime");
+	timespecadd(&c->message_timer, &ts, &c->message_timer);
+
+	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_STATUS;
+}
+
+void
+server_clear_client_message(struct client *c)
+{
+	if (c->message_string == NULL)
+		return;
+
+	xfree(c->message_string);
+	c->message_string = NULL;
+
+	c->tty.flags &= ~(TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_REDRAW;
+}
+
+void
+server_set_client_prompt(
+    struct client *c, const char *msg, void (*fn)(void *, char *), void *data)
+{
+	c->prompt_string = xstrdup(msg);
+
+	c->prompt_buffer = xstrdup("");
+	c->prompt_index = 0;
+
+	c->prompt_callback = fn;
+	c->prompt_data = data;
+
+	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_STATUS;
+}
+
+void
+server_clear_client_prompt(struct client *c)
+{
+	if (c->prompt_string == NULL)
+		return;
+
+	xfree(c->prompt_string);
+	c->prompt_string = NULL;
+
+	xfree(c->prompt_buffer);
+
+	c->tty.flags &= ~(TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_REDRAW;
+}
+
 void
 server_write_client(
     struct client *c, enum hdrtype type, const void *buf, size_t len)
@@ -146,51 +210,3 @@ server_status_window(struct window *w)
 			server_status_session(s);
 	}
 }
-
-void printflike2
-server_write_message(struct client *c, const char *fmt, ...)
-{
-	struct screen_redraw_ctx	ctx;
-	va_list				ap;
-	char			       *msg;
-	size_t				size;
-	u_int				slines;
-
-	slines = options_get_number(&c->session->options, "status-lines");
-
-	screen_redraw_start_client(&ctx, c);
-
-	va_start(ap, fmt);
-	xvasprintf(&msg, fmt, ap);
-	va_end(ap);
-
-	msg = xrealloc(msg, 1, c->sx + 1);
-	msg[c->sx] = '\0';
-
-	size = strlen(msg);
-	if (size < c->sx)
-		memset(msg + size, ' ', c->sx - size);
-
-	screen_redraw_move_cursor(&ctx, 0, c->sy - 1);
-	screen_redraw_set_attributes(&ctx, ATTR_REVERSE, 0x88);
-	screen_redraw_write_string(&ctx, "%s", msg);
-
-	buffer_flush(c->tty.fd, c->tty.in, c->tty.out);
-	usleep(750000);
-
-	memset(msg, ' ', c->sx);
-
-	screen_redraw_move_cursor(&ctx, 0, c->sy - 1);
-	screen_redraw_set_attributes(&ctx, 0, 0x88);
-	screen_redraw_write_string(&ctx, "%s", msg);
-
-	xfree(msg);
-
-	if (slines == 0) {
-		screen_redraw_lines(&ctx, c->sy - 1, 1);
-		screen_redraw_stop(&ctx);
-	} else {
-		screen_redraw_stop(&ctx);
-		server_status_client(c);
-	}
-}
