@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.65 2008/06/17 19:26:19 nicm Exp $ */
+/* $Id: server.c,v 1.77 2008/06/29 07:04:30 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,7 +24,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <paths.h>
 #include <poll.h>
 #include <signal.h>
 #include <stdio.h>
@@ -55,7 +54,7 @@ void		 server_handle_window(struct window *);
 void		 server_lost_client(struct client *);
 void	 	 server_lost_window(struct window *);
 void		 server_check_redraw(struct client *);
-void		 server_check_status(struct client *);
+void		 server_check_timers(struct client *);
 void		 server_update_socket(const char *);
 
 /* Fork new server. */
@@ -98,7 +97,7 @@ server_start(const char *path)
 	xmalloc_clear();
 #endif
 
-	/* 
+	/*
 	 * Must daemonise before loading configuration as the PID changes so
 	 * $TMUX would be wrong for sessions created in the config file.
 	 */
@@ -170,7 +169,7 @@ server_main(const char *srv_path, int srv_fd)
 	u_int		 i, n;
 
 	siginit();
- 
+
 	pfds = NULL;
 	while (!sigterm) {
 		/* Initialise pollfd array. */
@@ -189,7 +188,7 @@ server_main(const char *srv_path, int srv_fd)
 
 		/* Do the poll. */
 		log_debug("polling %d fds", nfds);
-		if ((nfds = poll(pfds, nfds, 500)) == -1) {
+		if ((nfds = poll(pfds, nfds, 100)) == -1) {
 			if (errno == EAGAIN || errno == EINTR)
 				continue;
 			fatal("poll failed");
@@ -264,7 +263,7 @@ server_fill_windows(struct pollfd **pfd)
 	u_int		 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
-		if ((w = ARRAY_ITEM(&windows, i)) == NULL)
+		if ((w = ARRAY_ITEM(&windows, i)) == NULL || w->fd == -1)
 			(*pfd)->fd = -1;
 		else {
 			(*pfd)->fd = w->fd;
@@ -285,7 +284,7 @@ server_handle_windows(struct pollfd **pfd)
 	u_int		 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
-		if ((w = ARRAY_ITEM(&windows, i)) != NULL) {
+		if ((w = ARRAY_ITEM(&windows, i)) != NULL && w->fd != -1) {
 			log_debug("testing window %d (%d)", (*pfd)->fd, w->fd);
 			if (buffer_poll(*pfd, w->in, w->out) != 0)
 				server_lost_window(w);
@@ -300,21 +299,40 @@ server_handle_windows(struct pollfd **pfd)
 void
 server_check_redraw(struct client *c)
 {
+	struct session		       *s;
 	struct screen_redraw_ctx	ctx;
 	struct screen			screen;
 	u_int				xx, yy, sx, sy;
+	char			        title[BUFSIZ];
+	int				flags;
 
 	if (c == NULL || c->session == NULL)
 		return;
+	s = c->session;
+
+	flags = c->tty.flags & TTY_FREEZE;
+	c->tty.flags &= ~TTY_FREEZE;
+
+	if (options_get_number(&s->options, "set-titles")) {
+		xsnprintf(title, sizeof title,
+		    "%s:%u:%s - \"%s\"", s->name, s->curw->idx,
+		    s->curw->window->name, s->curw->window->base.title);
+		if (c->title == NULL || strcmp(title, c->title) != 0) {
+			if (c->title != NULL)
+				xfree(c->title);
+			c->title = xstrdup(title);
+			tty_set_title(&c->tty, c->title);
+		}
+	}
 
 	xx = c->sx;
-	yy = c->sy - options_get_number(&global_options, "status-lines");
+	yy = c->sy - 1;
 	if (c->flags & CLIENT_REDRAW) {
-		sx = screen_size_x(c->session->curw->window->screen);
-		sy = screen_size_y(c->session->curw->window->screen);
+		sx = screen_size_x(s->curw->window->screen);
+		sy = screen_size_y(s->curw->window->screen);
 		if (sx < xx || sy < yy) {
 			/*
-			 * Fake up a blank(ish) screen and use it to draw the 
+			 * Fake up a blank(ish) screen and use it to draw the
 			 * empty regions. NOTE: because this uses
 			 * tty_write_client but doesn't write the client's
 			 * screen, this can't use anything which relies on
@@ -325,8 +343,8 @@ server_check_redraw(struct client *c)
 			if (sx < xx)
 				screen_redraw_columns(&ctx, sx, xx - sx);
 			if (sy < yy)  {
-				screen_fill_area(&screen, 
-				    0, sy, xx, 1, '-', 0, 0x70);
+				screen_fill_area(&screen,
+				    0, sy, xx, 1, '-', 0, 0x88);
 				screen_redraw_lines(&ctx, sy, yy - sy);
 			}
 			screen_redraw_stop(&ctx);
@@ -337,33 +355,50 @@ server_check_redraw(struct client *c)
 		screen_redraw_lines(&ctx, 0, screen_size_y(ctx.s));
 		screen_redraw_stop(&ctx);
 
-		status_redraw(c);
-	} else if (c->flags & CLIENT_STATUS)
-		status_redraw(c);
+		c->flags |= CLIENT_STATUS;
+	}
 
-	c->flags &= ~(CLIENT_CLEAR|CLIENT_REDRAW|CLIENT_STATUS);
+	if (c->flags & CLIENT_STATUS) {
+		if (c->message_string != NULL)
+			status_message_redraw(c);
+		else if (c->prompt_string != NULL)
+			status_prompt_redraw(c);
+		else
+			status_redraw(c);
+	}
+
+	c->tty.flags |= flags;
+
+	c->flags &= ~(CLIENT_REDRAW|CLIENT_STATUS);
 }
 
-/* Check for status line redraw on client. */
+/* Check for timers on client. */
 void
-server_check_status(struct client *c)
+server_check_timers(struct client *c)
 {
-	struct timespec	 ts;
-	u_int		 nlines, interval;
+	struct session	*s;
+	struct timespec	 ts, ts2;
+	u_int		 interval;
 
 	if (c == NULL || c->session == NULL)
 		return;
+	s = c->session;
 
-	nlines = options_get_number(&c->session->options, "status-lines");
-	interval = options_get_number(&c->session->options, "status-interval");
-	if (nlines == 0 || interval == 0)
-		return;
-	
 	if (clock_gettime(CLOCK_REALTIME, &ts) != 0)
 		fatal("clock_gettime");
-	ts.tv_sec -= interval;
 	
-	if (timespeccmp(&c->status_ts, &ts, <))
+	if (c->message_string != NULL && timespeccmp(&ts, &c->message_timer, >))
+		server_clear_client_message(c);
+
+	if (!options_get_number(&s->options, "status"))
+		return;
+	interval = options_get_number(&s->options, "status-interval");
+	if (interval == 0)
+		return;
+
+	memcpy(&ts2, &ts, sizeof ts2);
+	ts2.tv_sec -= interval;
+	if (timespeccmp(&c->status_timer, &ts2, <))
 		c->flags |= CLIENT_STATUS;
 }
 
@@ -377,8 +412,8 @@ server_fill_clients(struct pollfd **pfd)
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 
+		server_check_timers(c);
 		server_check_redraw(c);
-		server_check_status(c);
 
 		if (c == NULL)
 			(*pfd)->fd = -1;
@@ -414,7 +449,7 @@ server_handle_clients(struct pollfd **pfd)
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 
-		if (c != NULL) {		
+		if (c != NULL) {
 			log_debug("testing client %d (%d)", (*pfd)->fd, c->fd);
 			if (buffer_poll(*pfd, c->in, c->out) != 0) {
 				server_lost_client(c);
@@ -463,11 +498,18 @@ server_accept_client(int srv_fd)
 	c->out = buffer_create(BUFSIZ);
 
 	c->tty.fd = -1;
+	c->title = NULL;
 
 	c->session = NULL;
 	c->sx = 80;
 	c->sy = 25;
 
+	c->message_string = NULL;
+
+	c->prompt_string = NULL;
+	c->prompt_buffer = NULL;
+	c->prompt_index = 0;
+
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		if (ARRAY_ITEM(&clients, i) == NULL) {
 			ARRAY_SET(&clients, i, c);
@@ -485,8 +527,14 @@ server_handle_client(struct client *c)
 	struct window	*w = c->session->curw->window;
 	int		 key, prefix;
 
-	prefix = options_get_key(&c->session->options, "prefix-key");
+	prefix = options_get_number(&c->session->options, "prefix");
 	while (tty_keys_next(&c->tty, &key) == 0) {
+		server_clear_client_message(c);
+		if (c->prompt_string != NULL) {
+			status_prompt_key(c, key);
+			continue;
+		}
+
 		if (c->flags & CLIENT_PREFIX) {
 			key_bindings_dispatch(key, c);
 			c->flags &= ~CLIENT_PREFIX;
@@ -494,7 +542,7 @@ server_handle_client(struct client *c)
 		} else if (key == prefix)
 			c->flags |= CLIENT_PREFIX;
 		else
-			window_key(w, key);
+			window_key(w, c, key);
 	}
 }
 
@@ -581,6 +629,11 @@ server_lost_window(struct window *w)
 
 	log_debug("lost window %d", w->fd);
 
+	if (w->flags & WINDOW_ZOMBIFY) {
+		w->fd = -1;
+		return;
+	}
+
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
 		if (s == NULL)
