@@ -1,4 +1,4 @@
-/* $Id: session.c,v 1.37 2008/06/07 07:27:28 nicm Exp $ */
+/* $Id: session.c,v 1.41 2008/06/30 19:11:33 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -120,8 +120,9 @@ session_create(const char *name, const char *cmd, u_int sx, u_int sy)
 	s->curw = s->lastw = NULL;
 	RB_INIT(&s->windows);
 	TAILQ_INIT(&s->alerts);
+	paste_init_stack(&s->buffers);
 	options_init(&s->options, &global_options);
-	
+
 	s->sx = sx;
 	s->sy = sy;
 
@@ -165,6 +166,7 @@ session_destroy(struct session *s)
 
 	session_alert_cancel(s, NULL);
 	options_free(&s->options);
+	paste_free_stack(&s->buffers);
 
 	while (!RB_EMPTY(&s->windows))
 		winlink_remove(&s->windows, RB_ROOT(&s->windows));
@@ -189,7 +191,7 @@ struct winlink *
 session_new(struct session *s, const char *name, const char *cmd, int idx)
 {
 	struct window	*w;
-	const char	*env[] = { NULL, "TERM=screen", NULL };
+	const char	*env[] = { NULL /* TMUX= */, "TERM=screen", NULL };
 	char		 buf[256];
 	u_int		 i, hlimit;
 
@@ -201,6 +203,10 @@ session_new(struct session *s, const char *name, const char *cmd, int idx)
 	hlimit = options_get_number(&s->options, "history-limit");
 	if ((w = window_create(name, cmd, env, s->sx, s->sy, hlimit)) == NULL)
 		return (NULL);
+
+	if (options_get_number(&s->options, "remain-by-default"))
+		w->flags |= WINDOW_ZOMBIFY;
+	
 	return (session_attach(s, w, idx));
 }
 
