@@ -1,4 +1,4 @@
-/* $Id: tmux.c,v 1.56 2008/06/16 17:35:40 nicm Exp $ */
+/* $Id: tmux.c,v 1.71 2008/06/29 21:03:57 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -20,7 +20,6 @@
 #include <sys/wait.h>
 
 #include <errno.h>
-#include <paths.h>
 #include <poll.h>
 #include <pwd.h>
 #include <signal.h>
@@ -29,6 +28,10 @@
 #include <syslog.h>
 #include <unistd.h>
 
+#ifndef NO_PATHS_H
+#include <paths.h>
+#endif
+
 #include "tmux.h"
 
 #ifdef DEBUG
@@ -45,7 +48,6 @@ volatile sig_atomic_t sigterm;
 
 char		*cfg_file;
 struct options	 global_options;
-char		*paste_buffer;
 
 int		 debug_level;
 int		 be_quiet;
@@ -53,11 +55,15 @@ int		 be_quiet;
 void		 sighandler(int);
 __dead void	 usage(void);
 
+#ifdef NO_PROGNAME
+const char      *__progname = "tmux";
+#endif
+
 __dead void
 usage(void)
 {
-	fprintf(stderr, 
-	    "usage: %s [-v] [-f file] [-S socket-path] [command [flags]]\n",
+	fprintf(stderr,
+	    "usage: %s [-qVv] [-f file] [-S socket-path] [command [flags]]\n",
 	    __progname);
 	exit(1);
 }
@@ -170,12 +176,12 @@ main(int argc, char **argv)
 	struct hdr	 	 hdr;
 	const char		*shell;
 	struct passwd		*pw;
-	char			*client, *path, *name, *cause, *home;
+	char			*path, *cause, *home;
 	char			 rpath[MAXPATHLEN];
 	int	 		 n, opt;
 
-	client = path = name = NULL;
-        while ((opt = getopt(argc, argv, "f:S:qvV")) != EOF) {
+	path = NULL;
+        while ((opt = getopt(argc, argv, "f:qS:Vv")) != EOF) {
                 switch (opt) {
 		case 'f':
 			cfg_file = xstrdup(optarg);
@@ -203,17 +209,20 @@ main(int argc, char **argv)
 	siginit();
 
 	options_init(&global_options, NULL);
-	options_set_number(&global_options, "status-lines", 1);
-	options_set_colours(&global_options, "status-colour", 0x02);
+	options_set_number(&global_options, "status", 1);
+	options_set_number(&global_options, "status-fg", 0);
+	options_set_number(&global_options, "status-bg", 2);
 	options_set_number(&global_options, "bell-action", BELL_ANY);
 	options_set_number(&global_options, "history-limit", 2000);
-	options_set_key(&global_options, "prefix-key", META);
-	options_set_string(&global_options, "status-left", "");
+	options_set_number(&global_options, "display-time", 750);
+	options_set_number(&global_options, "prefix", META);
+	options_set_string(&global_options, "status-left", "%s", ""); /* ugh */
 	options_set_string(
 	    &global_options, "status-right", "%%H:%%M %%d-%%b-%%y");
 	options_set_number(&global_options, "status-interval", 15);
-
-	paste_buffer = NULL;
+	options_set_number(&global_options, "set-titles", 1);
+	options_set_number(&global_options, "buffer-limit", 9);
+	options_set_number(&global_options, "remain-by-default", 0);
 
 	if (cfg_file == NULL) {
 		home = getenv("HOME");
@@ -232,7 +241,7 @@ main(int argc, char **argv)
 		if (access(cfg_file, R_OK) != 0) {
 			log_warn("%s", cfg_file);
 			exit(1);
-		}		
+		}
 	}
 
 	if (path == NULL) {
