@@ -1,4 +1,4 @@
-/* $Id: tmux.h,v 1.144 2008/06/16 17:35:40 nicm Exp $ */
+/* $Id: tmux.h,v 1.172 2008/06/30 05:34:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -36,22 +36,31 @@
 #include "compat/tree.h"
 #endif
 
+#include <ncurses.h>
 #include <limits.h>
 #include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <term.h>
 
 #include "array.h"
 
-extern char    *__progname;
+extern const char    *__progname;
 
 #ifndef INFTIM
 #define INFTIM -1
 #endif
 
+#ifndef WAIT_ANY
+#define WAIT_ANY -1
+#endif
+
+#ifndef SUN_LEN
+#define SUN_LEN(sun) (sizeof (sun)->sun_path)
+#endif
+
 #ifndef __dead
 #define __dead __attribute__ ((__noreturn__))
 #endif
@@ -66,10 +75,28 @@ extern char    *__progname;
             ((tsp)->tv_sec cmp (usp)->tv_sec))
 #endif
 
+#ifndef timespecadd
+#define	timespecadd(tsp, usp, vsp)					\
+	do {								\
+		(vsp)->tv_sec = (tsp)->tv_sec + (usp)->tv_sec;		\
+		(vsp)->tv_nsec = (tsp)->tv_nsec + (usp)->tv_nsec;	\
+		if ((vsp)->tv_nsec >= 1000000000L) {			\
+			(vsp)->tv_sec++;				\
+			(vsp)->tv_nsec -= 1000000000L;			\
+		}							\
+	} while (0)
+#endif
+
 #ifndef TTY_NAME_MAX
 #define TTY_NAME_MAX 32
 #endif
 
+#ifdef NO_PATHS_H
+#define	_PATH_BSHELL	"/bin/sh"
+#define	_PATH_TMP	"/tmp/"
+#define _PATH_DEVNULL	"/dev/null"
+#endif
+
 /* Default configuration file. */
 #define DEFAULT_CFG ".tmux.conf"
 
@@ -118,157 +145,164 @@ struct buffer {
 #define BELL_CURRENT 2
 
 /* Key codes. ncurses defines KEY_*. Grrr. */
-#define KEYC_NONE 256
-#define KEYC_A1 -1
-#define KEYC_A3 -2
-#define KEYC_B2 -3
-/* #define KEYC_BACKSPACE -4 */
-#define KEYC_BEG -5
-#define KEYC_BTAB -6
-#define KEYC_C1 -7
-#define KEYC_C3 -8
-#define KEYC_CANCEL -9
-#define KEYC_CATAB -10
-#define KEYC_CLEAR -11
-#define KEYC_CLOSE -12
-#define KEYC_COMMAND -13
-#define KEYC_COPY -14
-#define KEYC_CREATE -15
-#define KEYC_CTAB -16
-#define KEYC_DC -17
-#define KEYC_DL -18
-#define KEYC_DOWN -19
-#define KEYC_EIC -20
-#define KEYC_END -21
-#define KEYC_ENTER -22
-#define KEYC_EOL -23
-#define KEYC_EOS -24
-#define KEYC_EXIT -25
-#define KEYC_F0 -26
-#define KEYC_F1 -27
-#define KEYC_F10 -28
-#define KEYC_F11 -29
-#define KEYC_F12 -30
-#define KEYC_F13 -31
-#define KEYC_F14 -32
-#define KEYC_F15 -33
-#define KEYC_F16 -34
-#define KEYC_F17 -35
-#define KEYC_F18 -36
-#define KEYC_F19 -37
-#define KEYC_F2 -38
-#define KEYC_F20 -39
-#define KEYC_F21 -40
-#define KEYC_F22 -41
-#define KEYC_F23 -42
-#define KEYC_F24 -43
-#define KEYC_F25 -44
-#define KEYC_F26 -45
-#define KEYC_F27 -46
-#define KEYC_F28 -47
-#define KEYC_F29 -48
-#define KEYC_F3 -49
-#define KEYC_F30 -50
-#define KEYC_F31 -51
-#define KEYC_F32 -52
-#define KEYC_F33 -53
-#define KEYC_F34 -54
-#define KEYC_F35 -55
-#define KEYC_F36 -56
-#define KEYC_F37 -57
-#define KEYC_F38 -58
-#define KEYC_F39 -59
-#define KEYC_F4 -60
-#define KEYC_F40 -61
-#define KEYC_F41 -62
-#define KEYC_F42 -63
-#define KEYC_F43 -64
-#define KEYC_F44 -65
-#define KEYC_F45 -66
-#define KEYC_F46 -67
-#define KEYC_F47 -68
-#define KEYC_F48 -69
-#define KEYC_F49 -70
-#define KEYC_F5 -71
-#define KEYC_F50 -72
-#define KEYC_F51 -73
-#define KEYC_F52 -74
-#define KEYC_F53 -75
-#define KEYC_F54 -76
-#define KEYC_F55 -77
-#define KEYC_F56 -78
-#define KEYC_F57 -79
-#define KEYC_F58 -80
-#define KEYC_F59 -81
-#define KEYC_F6 -82
-#define KEYC_F60 -83
-#define KEYC_F61 -84
-#define KEYC_F62 -85
-#define KEYC_F63 -86
-#define KEYC_F7 -87
-#define KEYC_F8 -88
-#define KEYC_F9 -89
-#define KEYC_FIND -90
-#define KEYC_HELP -91
-#define KEYC_HOME -92
-#define KEYC_IC -93
-#define KEYC_IL -94
-#define KEYC_LEFT -95
-#define KEYC_LL -96
-#define KEYC_MARK -97
-#define KEYC_MESSAGE -98
-#define KEYC_MOVE -99
-#define KEYC_NEXT -100
-#define KEYC_NPAGE -101
-#define KEYC_OPEN -102
-#define KEYC_OPTIONS -103
-#define KEYC_PPAGE -104
-#define KEYC_PREVIOUS -105
-#define KEYC_PRINT -106
-#define KEYC_REDO -107
-#define KEYC_REFERENCE -108
-#define KEYC_REFRESH -109
-#define KEYC_REPLACE -110
-#define KEYC_RESTART -111
-#define KEYC_RESUME -112
-#define KEYC_RIGHT -113
-#define KEYC_SAVE -114
-#define KEYC_SBEG -115
-#define KEYC_SCANCEL -116
-#define KEYC_SCOMMAND -117
-#define KEYC_SCOPY -118
-#define KEYC_SCREATE -119
-#define KEYC_SDC -120
-#define KEYC_SDL -121
-#define KEYC_SELECT -122
-#define KEYC_SEND -123
-#define KEYC_SEOL -124
-#define KEYC_SEXIT -125
-#define KEYC_SF -126
-#define KEYC_SFIND -127
-#define KEYC_SHELP -128
-#define KEYC_SHOME -129
-#define KEYC_SIC -130
-#define KEYC_SLEFT -131
-#define KEYC_SMESSAGE -132
-#define KEYC_SMOVE -133
-#define KEYC_SNEXT -134
-#define KEYC_SOPTIONS -135
-#define KEYC_SPREVIOUS -136
-#define KEYC_SPRINT -137
-#define KEYC_SR -138
-#define KEYC_SREDO -139
-#define KEYC_SREPLACE -140
-#define KEYC_SRIGHT -141
-#define KEYC_SRSUME -142
-#define KEYC_SSAVE -143
-#define KEYC_SSUSPEND -144
-#define KEYC_STAB -145
-#define KEYC_SUNDO -146
-#define KEYC_SUSPEND -147
-#define KEYC_UNDO -148
-#define KEYC_UP -149
-#define KEYC_MOUSE -150
+#define KEYC_NONE   0x0ffff
+#define KEYC_OFFSET 0x10000
+#define KEYC_ESCAPE 0x20000
+
+#define KEYC_ADDESCAPE(k) ((k) | KEYC_ESCAPE)
+#define KEYC_REMOVEESCAPE(k) ((k) & ~KEYC_ESCAPE)
+#define KEYC_ISESCAPE(k) ((k) != KEYC_NONE && ((k) & KEYC_ESCAPE))
+
+#define KEYC_A1 (KEYC_OFFSET + 0x1)
+#define KEYC_A3 (KEYC_OFFSET + 0x2)
+#define KEYC_B2 (KEYC_OFFSET + 0x3)
+/* #define KEYC_BACKSPACE (KEYC_OFFSET + 0x4) */
+#define KEYC_BEG (KEYC_OFFSET + 0x5)
+#define KEYC_BTAB (KEYC_OFFSET + 0x6)
+#define KEYC_C1 (KEYC_OFFSET + 0x7)
+#define KEYC_C3 (KEYC_OFFSET + 0x8)
+#define KEYC_CANCEL (KEYC_OFFSET + 0x9)
+#define KEYC_CATAB (KEYC_OFFSET + 0xa)
+#define KEYC_CLEAR (KEYC_OFFSET + 0xb)
+#define KEYC_CLOSE (KEYC_OFFSET + 0xc)
+#define KEYC_COMMAND (KEYC_OFFSET + 0xd)
+#define KEYC_COPY (KEYC_OFFSET + 0xe)
+#define KEYC_CREATE (KEYC_OFFSET + 0xf)
+#define KEYC_CTAB (KEYC_OFFSET + 0x10)
+#define KEYC_DC (KEYC_OFFSET + 0x11)
+#define KEYC_DL (KEYC_OFFSET + 0x12)
+#define KEYC_DOWN (KEYC_OFFSET + 0x13)
+#define KEYC_EIC (KEYC_OFFSET + 0x14)
+#define KEYC_END (KEYC_OFFSET + 0x15)
+#define KEYC_ENTER (KEYC_OFFSET + 0x16)
+#define KEYC_EOL (KEYC_OFFSET + 0x17)
+#define KEYC_EOS (KEYC_OFFSET + 0x18)
+#define KEYC_EXIT (KEYC_OFFSET + 0x19)
+#define KEYC_F0 (KEYC_OFFSET + 0x1a)
+#define KEYC_F1 (KEYC_OFFSET + 0x1b)
+#define KEYC_F10 (KEYC_OFFSET + 0x1c)
+#define KEYC_F11 (KEYC_OFFSET + 0x1d)
+#define KEYC_F12 (KEYC_OFFSET + 0x1e)
+#define KEYC_F13 (KEYC_OFFSET + 0x1f)
+#define KEYC_F14 (KEYC_OFFSET + 0x20)
+#define KEYC_F15 (KEYC_OFFSET + 0x21)
+#define KEYC_F16 (KEYC_OFFSET + 0x22)
+#define KEYC_F17 (KEYC_OFFSET + 0x23)
+#define KEYC_F18 (KEYC_OFFSET + 0x24)
+#define KEYC_F19 (KEYC_OFFSET + 0x25)
+#define KEYC_F2 (KEYC_OFFSET + 0x26)
+#define KEYC_F20 (KEYC_OFFSET + 0x27)
+#define KEYC_F21 (KEYC_OFFSET + 0x28)
+#define KEYC_F22 (KEYC_OFFSET + 0x29)
+#define KEYC_F23 (KEYC_OFFSET + 0x2a)
+#define KEYC_F24 (KEYC_OFFSET + 0x2b)
+#define KEYC_F25 (KEYC_OFFSET + 0x2c)
+#define KEYC_F26 (KEYC_OFFSET + 0x2d)
+#define KEYC_F27 (KEYC_OFFSET + 0x2e)
+#define KEYC_F28 (KEYC_OFFSET + 0x2f)
+#define KEYC_F29 (KEYC_OFFSET + 0x30)
+#define KEYC_F3 (KEYC_OFFSET + 0x31)
+#define KEYC_F30 (KEYC_OFFSET + 0x32)
+#define KEYC_F31 (KEYC_OFFSET + 0x33)
+#define KEYC_F32 (KEYC_OFFSET + 0x34)
+#define KEYC_F33 (KEYC_OFFSET + 0x35)
+#define KEYC_F34 (KEYC_OFFSET + 0x36)
+#define KEYC_F35 (KEYC_OFFSET + 0x37)
+#define KEYC_F36 (KEYC_OFFSET + 0x38)
+#define KEYC_F37 (KEYC_OFFSET + 0x39)
+#define KEYC_F38 (KEYC_OFFSET + 0x3a)
+#define KEYC_F39 (KEYC_OFFSET + 0x3b)
+#define KEYC_F4 (KEYC_OFFSET + 0x3c)
+#define KEYC_F40 (KEYC_OFFSET + 0x3d)
+#define KEYC_F41 (KEYC_OFFSET + 0x3e)
+#define KEYC_F42 (KEYC_OFFSET + 0x3f)
+#define KEYC_F43 (KEYC_OFFSET + 0x40)
+#define KEYC_F44 (KEYC_OFFSET + 0x41)
+#define KEYC_F45 (KEYC_OFFSET + 0x42)
+#define KEYC_F46 (KEYC_OFFSET + 0x43)
+#define KEYC_F47 (KEYC_OFFSET + 0x44)
+#define KEYC_F48 (KEYC_OFFSET + 0x45)
+#define KEYC_F49 (KEYC_OFFSET + 0x46)
+#define KEYC_F5 (KEYC_OFFSET + 0x47)
+#define KEYC_F50 (KEYC_OFFSET + 0x48)
+#define KEYC_F51 (KEYC_OFFSET + 0x49)
+#define KEYC_F52 (KEYC_OFFSET + 0x4a)
+#define KEYC_F53 (KEYC_OFFSET + 0x4b)
+#define KEYC_F54 (KEYC_OFFSET + 0x4c)
+#define KEYC_F55 (KEYC_OFFSET + 0x4d)
+#define KEYC_F56 (KEYC_OFFSET + 0x4e)
+#define KEYC_F57 (KEYC_OFFSET + 0x4f)
+#define KEYC_F58 (KEYC_OFFSET + 0x50)
+#define KEYC_F59 (KEYC_OFFSET + 0x51)
+#define KEYC_F6 (KEYC_OFFSET + 0x52)
+#define KEYC_F60 (KEYC_OFFSET + 0x53)
+#define KEYC_F61 (KEYC_OFFSET + 0x54)
+#define KEYC_F62 (KEYC_OFFSET + 0x55)
+#define KEYC_F63 (KEYC_OFFSET + 0x56)
+#define KEYC_F7 (KEYC_OFFSET + 0x57)
+#define KEYC_F8 (KEYC_OFFSET + 0x58)
+#define KEYC_F9 (KEYC_OFFSET + 0x59)
+#define KEYC_FIND (KEYC_OFFSET + 0x5a)
+#define KEYC_HELP (KEYC_OFFSET + 0x5b)
+#define KEYC_HOME (KEYC_OFFSET + 0x5c)
+#define KEYC_IC (KEYC_OFFSET + 0x5d)
+#define KEYC_IL (KEYC_OFFSET + 0x5e)
+#define KEYC_LEFT (KEYC_OFFSET + 0x5f)
+#define KEYC_LL (KEYC_OFFSET + 0x60)
+#define KEYC_MARK (KEYC_OFFSET + 0x61)
+#define KEYC_MESSAGE (KEYC_OFFSET + 0x62)
+#define KEYC_MOVE (KEYC_OFFSET + 0x63)
+#define KEYC_NEXT (KEYC_OFFSET + 0x64)
+#define KEYC_NPAGE (KEYC_OFFSET + 0x65)
+#define KEYC_OPEN (KEYC_OFFSET + 0x66)
+#define KEYC_OPTIONS (KEYC_OFFSET + 0x67)
+#define KEYC_PPAGE (KEYC_OFFSET + 0x68)
+#define KEYC_PREVIOUS (KEYC_OFFSET + 0x69)
+#define KEYC_PRINT (KEYC_OFFSET + 0x6a)
+#define KEYC_REDO (KEYC_OFFSET + 0x6b)
+#define KEYC_REFERENCE (KEYC_OFFSET + 0x6c)
+#define KEYC_REFRESH (KEYC_OFFSET + 0x6d)
+#define KEYC_REPLACE (KEYC_OFFSET + 0x6e)
+#define KEYC_RESTART (KEYC_OFFSET + 0x6f)
+#define KEYC_RESUME (KEYC_OFFSET + 0x70)
+#define KEYC_RIGHT (KEYC_OFFSET + 0x71)
+#define KEYC_SAVE (KEYC_OFFSET + 0x72)
+#define KEYC_SBEG (KEYC_OFFSET + 0x73)
+#define KEYC_SCANCEL (KEYC_OFFSET + 0x74)
+#define KEYC_SCOMMAND (KEYC_OFFSET + 0x75)
+#define KEYC_SCOPY (KEYC_OFFSET + 0x76)
+#define KEYC_SCREATE (KEYC_OFFSET + 0x77)
+#define KEYC_SDC (KEYC_OFFSET + 0x78)
+#define KEYC_SDL (KEYC_OFFSET + 0x79)
+#define KEYC_SELECT (KEYC_OFFSET + 0x7a)
+#define KEYC_SEND (KEYC_OFFSET + 0x7b)
+#define KEYC_SEOL (KEYC_OFFSET + 0x7c)
+#define KEYC_SEXIT (KEYC_OFFSET + 0x7d)
+#define KEYC_SF (KEYC_OFFSET + 0x7e)
+#define KEYC_SFIND (KEYC_OFFSET + 0x7f)
+#define KEYC_SHELP (KEYC_OFFSET + 0x80)
+#define KEYC_SHOME (KEYC_OFFSET + 0x81)
+#define KEYC_SIC (KEYC_OFFSET + 0x82)
+#define KEYC_SLEFT (KEYC_OFFSET + 0x83)
+#define KEYC_SMESSAGE (KEYC_OFFSET + 0x84)
+#define KEYC_SMOVE (KEYC_OFFSET + 0x85)
+#define KEYC_SNEXT (KEYC_OFFSET + 0x86)
+#define KEYC_SOPTIONS (KEYC_OFFSET + 0x87)
+#define KEYC_SPREVIOUS (KEYC_OFFSET + 0x88)
+#define KEYC_SPRINT (KEYC_OFFSET + 0x89)
+#define KEYC_SR (KEYC_OFFSET + 0x8a)
+#define KEYC_SREDO (KEYC_OFFSET + 0x8b)
+#define KEYC_SREPLACE (KEYC_OFFSET + 0x8c)
+#define KEYC_SRIGHT (KEYC_OFFSET + 0x8d)
+#define KEYC_SRSUME (KEYC_OFFSET + 0x8e)
+#define KEYC_SSAVE (KEYC_OFFSET + 0x8f)
+#define KEYC_SSUSPEND (KEYC_OFFSET + 0x90)
+#define KEYC_STAB (KEYC_OFFSET + 0x91)
+#define KEYC_SUNDO (KEYC_OFFSET + 0x92)
+#define KEYC_SUSPEND (KEYC_OFFSET + 0x93)
+#define KEYC_UNDO (KEYC_OFFSET + 0x94)
+#define KEYC_UP (KEYC_OFFSET + 0x95)
+#define KEYC_MOUSE (KEYC_OFFSET + 0x96)
 
 /* Output codes. */
 #define TTY_CHARACTER 0
@@ -295,9 +329,8 @@ struct buffer {
 #define TTY_KCURSORON 21
 #define TTY_KKEYPADOFF 22
 #define TTY_KKEYPADON 23
-#define TTY_TITLE 24
-#define TTY_MOUSEON 25
-#define TTY_MOUSEOFF 26 /* XXX merge allon/off into 1 arg? */
+#define TTY_MOUSEON 24
+#define TTY_MOUSEOFF 25 /* XXX merge allon/off into 1 arg? */
 
 /* Message codes. */
 enum hdrtype {
@@ -470,7 +503,7 @@ struct screen_write_ctx {
 /* Screen default contents. */
 #define SCREEN_DEFDATA ' '
 #define SCREEN_DEFATTR 0
-#define SCREEN_DEFCOLR 0x70	/* white on black */
+#define SCREEN_DEFCOLR 0x88	/* default on default ;-) */
 
 /* Input parser sequence argument. */
 struct input_arg {
@@ -505,16 +538,18 @@ struct input_ctx {
  * Window mode. Windows can be in several modes and this is used to call the
  * right function to handle input and output.
  */
+struct client;
 struct window_mode {
 	struct screen *(*init)(struct window *);
 	void	(*free)(struct window *);
 	void	(*resize)(struct window *, u_int, u_int);
-	void	(*key)(struct window *, int);
+	void	(*key)(struct window *, struct client *, int);
 };
 
 /* Window structure. */
 struct window {
 	char		*name;
+	char		*cmd;
 
 	int		 fd;
 	struct buffer	*in;
@@ -528,6 +563,7 @@ struct window {
 #define WINDOW_ACTIVITY 0x4
 #define WINDOW_MONITOR 0x8
 #define WINDOW_AGGRESSIVE 0x10
+#define WINDOW_ZOMBIFY 0x20
 
 	u_int		 limitx;
 	u_int		 limity;
@@ -576,6 +612,13 @@ struct options {
 	struct options	*parent;
 };
 
+/* Paste buffer. */
+struct paste_buffer {
+     	char		*data;
+	struct timespec	 ts;
+};
+ARRAY_DECL(paste_stack, struct paste_buffer *);
+
 /* Client session. */
 struct session_alert {
 	struct winlink	*wl;
@@ -597,6 +640,8 @@ struct session {
 
 	struct options	 options;
 
+	struct paste_stack buffers;
+
 	TAILQ_HEAD(, session_alert) alerts;
 
 #define SESSION_UNATTACHED 0x1	/* not attached to any clients */
@@ -637,6 +682,13 @@ struct tty {
 
 	u_char		 acs[UCHAR_MAX + 1];
 
+#define TTY_NOCURSOR 0x1
+#define TTY_FREEZE 0x2
+#define TTY_ESCAPE 0x4
+	int		 flags;
+
+	struct timespec	 key_timer;
+
 	size_t		 ksize;	/* maximum key size */
 	RB_HEAD(tty_keys, tty_key) ktree;
 };
@@ -647,8 +699,10 @@ struct client {
 	struct buffer	*in;
 	struct buffer	*out;
 
+	char		*title;
+
 	struct tty 	 tty;
-	struct timespec	 status_ts;
+	struct timespec	 status_timer;
 
 	u_int		 sx;
 	u_int		 sy;
@@ -658,9 +712,17 @@ struct client {
 #define CLIENT_MOUSE 0x4
 #define CLIENT_REDRAW 0x8
 #define CLIENT_STATUS 0x10
-#define CLIENT_CLEAR 0x20
 	int		 flags;
 
+	char		*message_string;
+	struct timespec	 message_timer;
+
+	char		*prompt_string;
+	char		*prompt_buffer;
+	size_t		 prompt_index;
+	void		 (*prompt_callback)(void *, char *);
+	void		*prompt_data;
+
 	struct session	*session;
 };
 ARRAY_DECL(clients, struct client *);
@@ -687,9 +749,6 @@ struct cmd_ctx {
 	void		(*print)(struct cmd_ctx *, const char *, ...);
 	void		(*info)(struct cmd_ctx *, const char *, ...);
 	void		(*error)(struct cmd_ctx *, const char *, ...);
-
-#define CMD_KEY 0x1
-	int		flags;
 };
 
 struct cmd {
@@ -707,6 +766,7 @@ struct cmd_entry {
 #define CMD_KFLAG 0x4
 #define CMD_DFLAG 0x8
 #define CMD_ONEARG 0x10
+#define CMD_ZEROONEARG 0x20
 	int		 flags;
 
 	void		 (*init)(struct cmd *, int);
@@ -732,6 +792,13 @@ struct cmd_srcdst_data {
 	char	*arg;
 };
 
+struct cmd_buffer_data {
+	int	 flags;
+	char	*target;
+	int	 buffer;
+	char	*arg;
+};
+
 /* Key binding. */
 struct binding {
 	int		 key;
@@ -739,6 +806,26 @@ struct binding {
 };
 ARRAY_DECL(bindings, struct binding *);
 
+/* Set/display option data. */
+struct set_option_entry {
+	const char	*name;
+	enum {
+		SET_OPTION_STRING,
+		SET_OPTION_NUMBER,
+		SET_OPTION_KEY,		
+		SET_OPTION_COLOUR,
+		SET_OPTION_FLAG,
+		SET_OPTION_CHOICE
+	} type;
+
+	u_int		 minimum;
+	u_int		 maximum;
+	
+	const char     **choices;
+};
+extern const struct set_option_entry set_option_table[];
+#define NSETOPTION 14
+
 #ifdef NO_STRTONUM
 /* strtonum.c */
 long long	 strtonum(const char *, long long, long long, const char **);
@@ -754,12 +841,32 @@ size_t	 	 strlcpy(char *, const char *, size_t);
 size_t	 	 strlcat(char *, const char *, size_t);
 #endif
 
+#ifdef NO_DAEMON
+/* daemon.c */
+int	 	 daemon(int, int);
+#endif
+
+#ifdef NO_FORKPTY
+/* forkpty.c */
+pid_t		 forkpty(int *, char *, struct termios *, struct winsize *);
+#endif
+
+#ifdef NO_ASPRINTF
+/* asprintf.c */
+int	asprintf(char **, const char *, ...);
+int	vasprintf(char **, const char *, va_list);
+#endif
+
+#ifdef NO_FGETLN
+/* fgetln.c */
+char   *fgetln(FILE *, size_t *);
+#endif
+
 /* tmux.c */
 extern volatile sig_atomic_t sigwinch;
 extern volatile sig_atomic_t sigterm;
 extern struct options global_options;
 extern char	*cfg_file;
-extern char	*paste_buffer;
 extern int	 debug_level;
 extern int	 be_quiet;
 void		 logfile(const char *);
@@ -774,18 +881,17 @@ int	options_cmp(struct options_entry *, struct options_entry *);
 SPLAY_PROTOTYPE(options_tree, options_entry, entry, options_cmp);
 void	options_init(struct options *, struct options *);
 void	options_free(struct options *);
+struct options_entry *options_find1(struct options *, const char *);
+struct options_entry *options_find(struct options *, const char *);
 void printflike3 options_set_string(
     	    struct options *, const char *, const char *, ...);
 char   *options_get_string(struct options *, const char *);
 void	options_set_number(struct options *, const char *, long long);
 long long options_get_number(struct options *, const char *);
-void	options_set_key(struct options *, const char *, int);
-int	options_get_key(struct options *, const char *);
-void	options_set_colours(struct options *, const char *, u_char);
-u_char	options_get_colours(struct options *, const char *);
 
 /* tty.c */
 void		 tty_init(struct tty *, char *, char *);
+void		 tty_set_title(struct tty *, const char *);
 int		 tty_open(struct tty *, char **);
 void		 tty_close(struct tty *);
 void		 tty_free(struct tty *);
@@ -806,12 +912,24 @@ void		 tty_vwrite_window(void *, int, va_list);
 void		 tty_write_session(void *, int, ...);
 void		 tty_vwrite_session(void *, int, va_list);
 
+/* paste.c */
+void		 paste_init_stack(struct paste_stack *);
+void		 paste_free_stack(struct paste_stack *);
+struct paste_buffer *paste_walk_stack(struct paste_stack *, uint *);
+struct paste_buffer *paste_get_top(struct paste_stack *);
+struct paste_buffer *paste_get_index(struct paste_stack *, u_int);
+int	     	 paste_free_top(struct paste_stack *);
+int		 paste_free_index(struct paste_stack *, u_int);
+void		 paste_add(struct paste_stack *, const char *, u_int);
+int		 paste_replace(struct paste_stack *, u_int, const char *);
+
 /* arg.c */
 struct client 	*arg_parse_client(const char *);
 struct session 	*arg_parse_session(const char *);
 int		 arg_parse_window(const char *, struct session **, int *);
 
 /* cmd.c */
+char		*cmd_complete(const char *);
 struct cmd	*cmd_parse(int, char **, char **);
 void		 cmd_exec(struct cmd *, struct cmd_ctx *);
 void		 cmd_send(struct cmd *, struct buffer *);
@@ -824,9 +942,12 @@ struct client	*cmd_find_client(struct cmd_ctx *, const char *);
 struct session	*cmd_find_session(struct cmd_ctx *, const char *);
 struct winlink	*cmd_find_window(
     		     struct cmd_ctx *, const char *, struct session **);
+extern const struct cmd_entry *cmd_table[];
 extern const struct cmd_entry cmd_attach_session_entry;
 extern const struct cmd_entry cmd_bind_key_entry;
+extern const struct cmd_entry cmd_command_prompt_entry;
 extern const struct cmd_entry cmd_copy_mode_entry;
+extern const struct cmd_entry cmd_delete_buffer_entry;
 extern const struct cmd_entry cmd_detach_client_entry;
 extern const struct cmd_entry cmd_has_session_entry;
 extern const struct cmd_entry cmd_kill_server_entry;
@@ -834,10 +955,13 @@ extern const struct cmd_entry cmd_kill_session_entry;
 extern const struct cmd_entry cmd_kill_window_entry;
 extern const struct cmd_entry cmd_last_window_entry;
 extern const struct cmd_entry cmd_link_window_entry;
+extern const struct cmd_entry cmd_list_buffers_entry;
 extern const struct cmd_entry cmd_list_clients_entry;
+extern const struct cmd_entry cmd_list_commands_entry;
 extern const struct cmd_entry cmd_list_keys_entry;
 extern const struct cmd_entry cmd_list_sessions_entry;
 extern const struct cmd_entry cmd_list_windows_entry;
+extern const struct cmd_entry cmd_move_window_entry;
 extern const struct cmd_entry cmd_new_session_entry;
 extern const struct cmd_entry cmd_new_window_entry;
 extern const struct cmd_entry cmd_next_window_entry;
@@ -846,12 +970,16 @@ extern const struct cmd_entry cmd_previous_window_entry;
 extern const struct cmd_entry cmd_refresh_client_entry;
 extern const struct cmd_entry cmd_rename_session_entry;
 extern const struct cmd_entry cmd_rename_window_entry;
+extern const struct cmd_entry cmd_respawn_window_entry;
 extern const struct cmd_entry cmd_scroll_mode_entry;
 extern const struct cmd_entry cmd_select_window_entry;
 extern const struct cmd_entry cmd_send_keys_entry;
 extern const struct cmd_entry cmd_send_prefix_entry;
+extern const struct cmd_entry cmd_select_prompt_entry;
+extern const struct cmd_entry cmd_set_buffer_entry;
 extern const struct cmd_entry cmd_set_option_entry;
 extern const struct cmd_entry cmd_set_window_option_entry;
+extern const struct cmd_entry cmd_show_buffer_entry;
 extern const struct cmd_entry cmd_show_options_entry;
 extern const struct cmd_entry cmd_show_window_options_entry;
 extern const struct cmd_entry cmd_start_server_entry;
@@ -860,6 +988,9 @@ extern const struct cmd_entry cmd_switch_client_entry;
 extern const struct cmd_entry cmd_unbind_key_entry;
 extern const struct cmd_entry cmd_unlink_window_entry;
 
+/* cmd-string.c */
+struct cmd *cmd_string_parse(const char *, char **);
+
 /* cmd-generic.c */
 #define CMD_TARGET_WINDOW_USAGE "[-t target-window]"
 #define CMD_TARGET_SESSION_USAGE "[-t target-session]"
@@ -881,6 +1012,16 @@ void	cmd_srcdst_send(struct cmd *, struct buffer *);
 void	cmd_srcdst_recv(struct cmd *, struct buffer *);
 void	cmd_srcdst_free(struct cmd *);
 void	cmd_srcdst_print(struct cmd *, char *, size_t);
+#define CMD_BUFFER_WINDOW_USAGE "[-b buffer-index] [-t target-window]"
+#define CMD_BUFFER_SESSION_USAGE "[-b buffer-index] [-t target-session]"
+#define CMD_BUFFER_CLIENT_USAGE "[-b buffer-index] [-t target-client]"
+void	cmd_buffer_init(struct cmd *, int);
+int	cmd_buffer_parse(struct cmd *, int, char **, char **);
+void	cmd_buffer_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_buffer_send(struct cmd *, struct buffer *);
+void	cmd_buffer_recv(struct cmd *, struct buffer *);
+void	cmd_buffer_free(struct cmd *);
+void	cmd_buffer_print(struct cmd *, char *, size_t);
 
 /* client.c */
 int	 client_init(const char *, struct client_ctx *, int);
@@ -903,6 +1044,9 @@ void	 key_bindings_remove(int);
 void	 key_bindings_init(void);
 void	 key_bindings_free(void);
 void	 key_bindings_dispatch(int, struct client *);
+void printflike2 key_bindings_error(struct cmd_ctx *, const char *, ...);
+void printflike2 key_bindings_print(struct cmd_ctx *, const char *, ...);
+void printflike2 key_bindings_info(struct cmd_ctx *, const char *, ...);
 
 /* key-string.c */
 int	 key_string_lookup_string(const char *);
@@ -916,6 +1060,11 @@ int	 server_start(const char *);
 int	 server_msg_dispatch(struct client *);
 
 /* server-fn.c */
+void	 server_set_client_message(struct client *, const char *);
+void	 server_clear_client_message(struct client *);
+void	 server_set_client_prompt(
+	     struct client *, const char *, void (*)(void *, char *), void *);
+void	 server_clear_client_prompt(struct client *);
 struct session *server_extract_session(
     	     struct msg_command_data *, char *, char **);
 void	 server_write_client(
@@ -930,10 +1079,12 @@ void	 server_redraw_session(struct session *);
 void	 server_status_session(struct session *);
 void	 server_redraw_window(struct window *);
 void	 server_status_window(struct window *);
-void printflike2 server_write_message(struct client *, const char *, ...);
 
 /* status.c */
-void	 status_redraw(struct client *c);
+void	 status_redraw(struct client *);
+void	 status_message_redraw(struct client *);
+void	 status_prompt_redraw(struct client *);
+void	 status_prompt_key(struct client *, int);
 
 /* resize.c */
 void	 recalculate_sizes(void);
@@ -1024,6 +1175,7 @@ void	screen_redraw_columns(struct screen_redraw_ctx *, u_int, u_int);
 const char *screen_colourstring(u_char);
 u_char	 screen_stringcolour(const char *);
 void	 screen_create(struct screen *, u_int, u_int, u_int);
+void	 screen_reset(struct screen *);
 void	 screen_destroy(struct screen *);
 void	 screen_resize(struct screen *, u_int, u_int);
 void	 screen_expand_line(struct screen *, u_int, u_int);
@@ -1054,13 +1206,14 @@ void		 winlink_remove(struct winlinks *, struct winlink *);
 struct winlink	*winlink_next(struct winlinks *, struct winlink *);
 struct winlink	*winlink_previous(struct winlinks *, struct winlink *);
 struct window	*window_create(const char *,
-		     const char *, const char **, u_int, u_int, u_int);
+    const char *, const char **, u_int, u_int, u_int);
+int		 window_spawn(struct window *, const char *, const char **);
 void		 window_destroy(struct window *);
 int		 window_resize(struct window *, u_int, u_int);
 int		 window_set_mode(struct window *, const struct window_mode *);
 void		 window_reset_mode(struct window *);
 void		 window_parse(struct window *);
-void		 window_key(struct window *, int);
+void		 window_key(struct window *, struct client *, int);
 
 /* window-copy.c */
 extern const struct window_mode window_copy_mode;
