@@ -1,4 +1,4 @@
-/* $Id: tty.c,v 1.23 2008/06/10 18:51:22 nicm Exp $ */
+/* $Id: tty.c,v 1.35 2008/06/23 16:58:49 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,14 +19,12 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
-#include <curses.h>
+#include <ncurses.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
-#define TTYDEFCHARS
-/* glibc requires unistd.h before termios.h for TTYDEFCHARS. */
-#include <unistd.h>
 #include <termios.h>
+#include <unistd.h>
 
 #include "tmux.h"
 
@@ -36,6 +34,7 @@ void 	tty_free_term(struct tty_term *);
 void	tty_fill_acs(struct tty *);
 u_char	tty_get_acs(struct tty *, u_char);
 
+const char *tty_strip(const char *);
 void	tty_raw(struct tty *, const char *);
 void	tty_puts(struct tty *, const char *);
 void	tty_putc(struct tty *, char);
@@ -59,13 +58,18 @@ int
 tty_open(struct tty *tty, char **cause)
 {
 	struct termios	 tio;
-	int		 what;
+	int		 what, mode;
 
 	tty->fd = open(tty->path, O_RDWR|O_NONBLOCK);
 	if (tty->fd == -1) {
 		xasprintf(cause, "%s: %s", tty->path, strerror(errno));
 		return (-1);
 	}
+	
+	if ((mode = fcntl(tty->fd, F_GETFL)) == -1)
+		fatal("fcntl");
+	if (fcntl(tty->fd, F_SETFL, mode|O_NONBLOCK) == -1)
+		fatal("fcntl");
 
 	if ((tty->term = tty_find_term(tty->termname, tty->fd, cause)) == NULL)
 		goto error;
@@ -74,32 +78,19 @@ tty_open(struct tty *tty, char **cause)
 	tty->out = buffer_create(BUFSIZ);
 
 	tty->attr = 0;
-	tty->colr = 0x70;
+	tty->colr = 0x88;
 
-	tty_keys_init(tty);
-
-	tty_fill_acs(tty);
+	tty->flags = 0;
 
 	if (tcgetattr(tty->fd, &tty->tio) != 0)
 		fatal("tcgetattr failed");
-#if 0
-	/*
-	 * This fails on some Linuxes with EINVAL. Which is weird since
-	 * all we do is take stuff out of the defaults...
-	 */
-	memset(&tio, 0, sizeof tio);
-	tio.c_iflag = TTYDEF_IFLAG & ~(IXON|IXOFF|ICRNL|INLCR);
-	tio.c_oflag = TTYDEF_OFLAG & ~(OPOST|ONLCR|OCRNL|ONLRET);
-	tio.c_lflag =
-	    TTYDEF_LFLAG & ~(IEXTEN|ICANON|ECHO|ECHOE|ECHOKE|ECHOCTL|ISIG);
-	tio.c_cflag = TTYDEF_CFLAG;
-	memcpy(&tio.c_cc, ttydefchars, sizeof tio.c_cc);
-	cfsetspeed(&tio, TTYDEF_SPEED);
-#endif
 	memcpy(&tio, &tty->tio, sizeof tio);
-	tio.c_iflag &= ~(IXON|IXOFF|ICRNL|INLCR);
-	tio.c_oflag &= ~(OPOST|ONLCR|OCRNL|ONLRET);
-	tio.c_lflag &= ~(IEXTEN|ICANON|ECHO|ECHOE|ECHOKE|ECHOCTL|ISIG);
+	tio.c_iflag &= ~(IXON|IXOFF|ICRNL|INLCR|IGNCR|IMAXBEL|IUCLC|ISTRIP);
+	tio.c_iflag |= IGNBRK;
+	tio.c_oflag &= ~(OPOST|ONLCR|OCRNL|ONLRET|OLCUC);
+	tio.c_lflag &= ~(IEXTEN|ICANON|ECHO|ECHOE|ECHONL|ECHOCTL|ECHOPRT|ECHOKE|ECHOCTL|ISIG);
+	tio.c_cc[VMIN] = 1;
+        tio.c_cc[VTIME] = 0;
 	if (tcsetattr(tty->fd, TCSANOW, &tio) != 0)
 		fatal("tcsetattr failed");
 
@@ -109,6 +100,13 @@ tty_open(struct tty *tty, char **cause)
 		fatal("ioctl(TIOCFLUSH)");
 #endif
 
+	if (init_1string != NULL)
+		tty_puts(tty, init_1string);
+	if (init_2string != NULL)
+		tty_puts(tty, init_2string);
+	if (init_3string != NULL)
+		tty_puts(tty, init_3string);
+
 	if (enter_ca_mode != NULL)
 		tty_puts(tty, enter_ca_mode);
 	if (keypad_xmit != NULL)
@@ -117,6 +115,10 @@ tty_open(struct tty *tty, char **cause)
 		tty_puts(tty, ena_acs);
 	tty_puts(tty, clear_screen);
 
+	tty_keys_init(tty);
+
+	tty_fill_acs(tty);
+
 	return (0);
 
 error:
@@ -144,7 +146,7 @@ tty_close(struct tty *tty)
 	} else {
 		 if (tcsetattr(tty->fd, TCSANOW, &tty->tio) != 0)
 			 fatal("tcsetattr failed");
-		 
+
 		 tty_raw(tty, tparm(change_scroll_region, 0, ws.ws_row - 1));
 		 if (keypad_local != NULL)
 			 tty_raw(tty, keypad_local);
@@ -183,7 +185,7 @@ tty_find_term(char *name, int fd, char **cause)
 {
 	struct tty_term	*term;
 	int		 error;
-	
+
 	TAILQ_FOREACH(term, &tty_terms, entry) {
 		if (strcmp(term->name, name) == 0) {
 			term->references++;
@@ -322,16 +324,51 @@ tty_get_acs(struct tty *tty, u_char ch)
 	return (ch);
 }
 
+const char *
+tty_strip(const char *s)
+{
+	const char     *ptr;
+	static char	buf[BUFSIZ];
+	size_t		len;
+
+	/* Ignore strings with no padding. */
+	if (strchr(s, '$') == NULL)
+		return (s);
+
+	len = 0;
+	for (ptr = s; *ptr != '\0'; ptr++) {
+		if (*ptr == '$' && *(ptr + 1) == '<') {
+			while (*ptr != '\0' && *ptr != '>')
+				ptr++;
+			if (*ptr == '>')
+				ptr++;
+		}
+		
+		buf[len++] = *ptr;
+		if (len == (sizeof buf) - 1)
+			break;
+	}
+	buf[len] = '\0';
+	
+	return (buf);
+}
+
 void
 tty_raw(struct tty *tty, const char *s)
 {
-	write(tty->fd, s, strlen(s));
+	const char	*t;
+
+	t = tty_strip(s);
+	write(tty->fd, t, strlen(t));
 }
 
 void
 tty_puts(struct tty *tty, const char *s)
 {
-	buffer_write(tty->out, s, strlen(s));
+	const char	*t;
+
+	t = tty_strip(s);
+	buffer_write(tty->out, t, strlen(t));
 }
 
 void
@@ -342,12 +379,28 @@ tty_putc(struct tty *tty, char ch)
 	buffer_write8(tty->out, ch);
 }
 
+void
+tty_set_title(struct tty *tty, const char *title)
+{
+	if (strstr(tty->termname, "xterm") == NULL &&
+	    strstr(tty->termname, "rxvt") == NULL &&
+	    strcmp(tty->termname, "screen") != 0)
+		return;
+
+	tty_puts(tty, "\033]0;");
+	tty_puts(tty, title);
+	tty_putc(tty, '\007');
+}
+
 void
 tty_vwrite(struct tty *tty, struct screen *s, int cmd, va_list ap)
 {
 	char	ch;
 	u_int	i, ua, ub;
 
+	if (tty->flags & TTY_FREEZE)
+		return;
+	
 	if (tty->term == NULL) /* XXX XXX */
 		return;
 	set_curterm(tty->term->term);
@@ -357,7 +410,7 @@ tty_vwrite(struct tty *tty, struct screen *s, int cmd, va_list ap)
 		ch = va_arg(ap, int);
 		switch (ch) {
 		case '\n':	/* LF */
-			tty_puts(tty, cursor_down);
+			tty_putc(tty, '\n');
 			break;
 		case '\r':	/* CR */
 			tty_puts(tty, carriage_return);
@@ -493,7 +546,7 @@ tty_vwrite(struct tty *tty, struct screen *s, int cmd, va_list ap)
 		}
 		break;
 	case TTY_CURSORON:
-		if (cursor_normal != NULL)
+		if (!(tty->flags & TTY_NOCURSOR) && cursor_normal != NULL)
 			tty_puts(tty, cursor_normal);
 		break;
 	case TTY_CURSOROFF:
@@ -538,13 +591,11 @@ tty_vwrite(struct tty *tty, struct screen *s, int cmd, va_list ap)
 #endif
 	case TTY_MOUSEOFF:
 		if (key_mouse != NULL)
-			tty_puts(tty, "\e[?1000l");
+			tty_puts(tty, "\033[?1000l");
 		break;
 	case TTY_MOUSEON:
 		if (key_mouse != NULL)
-			tty_puts(tty, "\e[?1000h");
-		break;
-	case TTY_TITLE:
+			tty_puts(tty, "\033[?1000h");
 		break;
 	case TTY_ATTRIBUTES:
 		ua = va_arg(ap, u_int);
@@ -568,7 +619,7 @@ tty_attributes(struct tty *tty, u_char attr, u_char colr)
 		    exit_alt_charset_mode != NULL)
 			tty_puts(tty, exit_alt_charset_mode);
 		tty_puts(tty, exit_attribute_mode);
-		tty->colr = 0x70;
+		tty->colr = 0x88;
 		tty->attr = 0;
 	}
 
@@ -595,32 +646,22 @@ tty_attributes(struct tty *tty, u_char attr, u_char colr)
 
 	fg = (colr >> 4) & 0xf;
 	if (fg != ((tty->colr >> 4) & 0xf)) {
-		if (tigetflag("AX") == TRUE) {
-			if (fg == 7)
-				fg = 8;
-		} else {
-			if (fg == 8)
-				fg = 7;
-		}
-
+		if (tigetflag("AX") != TRUE && fg == 8)
+			fg = 7;
+		
 		if (fg == 8)
-			tty_puts(tty, "\e[39m");
+			tty_puts(tty, "\033[39m");
 		else if (set_a_foreground != NULL)
 			tty_puts(tty, tparm(set_a_foreground, fg));
 	}
 
 	bg = colr & 0xf;
 	if (bg != (tty->colr & 0xf)) {
-		if (tigetflag("AX") == TRUE) {
-			if (bg == 0)
-				bg = 8;
-		} else {
-			if (bg == 8)
-				bg = 0;
-		}
+		if (tigetflag("AX") != TRUE && bg == 8)
+			bg = 0;
 
 		if (bg == 8)
-			tty_puts(tty, "\e[49m");
+			tty_puts(tty, "\033[49m");
 		else if (set_a_background != NULL)
 			tty_puts(tty, tparm(set_a_background, bg));
 	}
