@@ -1,4 +1,4 @@
-/* $Id: window-copy.c,v 1.17 2008/06/04 18:50:35 nicm Exp $ */
+/* $Id: window-copy.c,v 1.25 2008/06/27 17:41:48 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -25,7 +25,7 @@
 struct screen *window_copy_init(struct window *);
 void	window_copy_free(struct window *);
 void	window_copy_resize(struct window *, u_int, u_int);
-void	window_copy_key(struct window *, int);
+void	window_copy_key(struct window *, struct client *, int);
 
 void	window_copy_redraw_lines(struct window *, u_int, u_int);
 void	window_copy_redraw_screen(struct window *);
@@ -41,7 +41,7 @@ void	window_copy_write_columns(
 void	window_copy_update_cursor(struct window *);
 void	window_copy_start_selection(struct window *);
 int	window_copy_update_selection(struct window *);
-void	window_copy_copy_selection(struct window *);
+void	window_copy_copy_selection(struct window *, struct client *);
 void	window_copy_copy_line(
 	    struct window *, char **, size_t *, size_t *, u_int, u_int, u_int);
 u_int	window_copy_find_length(struct window *, u_int);
@@ -51,11 +51,17 @@ void	window_copy_cursor_left(struct window *);
 void	window_copy_cursor_right(struct window *);
 void	window_copy_cursor_up(struct window *);
 void	window_copy_cursor_down(struct window *);
+void	window_copy_cursor_next_word(struct window *);
+void	window_copy_cursor_previous_word(struct window *);
 void	window_copy_scroll_left(struct window *, u_int);
 void	window_copy_scroll_right(struct window *, u_int);
 void	window_copy_scroll_up(struct window *, u_int);
 void	window_copy_scroll_down(struct window *, u_int);
 
+const char *space_characters = " -_@";
+#define window_copy_is_space(w, x, y) \
+	(strchr(space_characters, (w)->base.grid_data[y][x]) != NULL)
+
 const struct window_mode window_copy_mode = {
 	window_copy_init,
 	window_copy_free,
@@ -71,7 +77,7 @@ struct window_copy_mode_data {
 
 	u_int	selx;
 	u_int	sely;
-	
+
 	u_int	cx;
 	u_int	cy;
 };
@@ -126,7 +132,7 @@ window_copy_resize(struct window *w, u_int sx, u_int sy)
 }
 
 void
-window_copy_key(struct window *w, int key)
+window_copy_key(struct window *w, struct client *c, int key)
 {
 	struct window_copy_mode_data	*data = w->modedata;
 	struct screen			*s = &data->screen;
@@ -180,9 +186,11 @@ window_copy_key(struct window *w, int key)
 		screen_clear_selection(&data->screen);
 		break;
 	case '\027':	/* C-w */
-	case '\r':	/* enter */
-		window_copy_copy_selection(w);
-		window_reset_mode(w);
+ 	case '\r':	/* enter */
+		if (c != NULL && c->session != NULL) {
+			window_copy_copy_selection(w, c);
+			window_reset_mode(w);
+		}
 		break;
 	case '0':
 	case '\001':	/* C-a */
@@ -192,6 +200,16 @@ window_copy_key(struct window *w, int key)
 	case '\005':	/* C-e */
 		window_copy_cursor_end_of_line(w);
 		break;
+	case 'w':
+	case KEYC_ADDESCAPE('F'):	/* M-F */
+	case KEYC_ADDESCAPE('f'):	/* M-f */
+		window_copy_cursor_next_word(w);
+		break;
+	case 'b':
+	case KEYC_ADDESCAPE('B'):	/* M-B */
+	case KEYC_ADDESCAPE('b'):	/* M-b */
+		window_copy_cursor_previous_word(w);
+		break;
 	}
 }
 
@@ -205,7 +223,7 @@ window_copy_write_line(
 
 	if (py == 0) {
 		screen_write_set_attributes(
-		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x70);
+		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x88);
 		screen_write_move_cursor(ctx, 0, 0);
 		size = screen_write_put_string_rjust(
 		    ctx, "[%u,%u/%u]", data->ox, data->oy, w->base.hsize);
@@ -354,13 +372,13 @@ window_copy_update_selection(struct window *w)
 }
 
 void
-window_copy_copy_selection(struct window *w)
+window_copy_copy_selection(struct window *w, struct client *c)
 {
 	struct window_copy_mode_data	*data = w->modedata;
 	struct screen			*s = &data->screen;
 	char				*buf;
 	size_t	 			 len, off;
-	u_int	 			 i, xx, yy, sx, sy, ex, ey;
+	u_int	 			 i, xx, yy, sx, sy, ex, ey, limit;
 
 	if (!s->sel.flag)
 		return;
@@ -412,9 +430,10 @@ window_copy_copy_selection(struct window *w)
 	if (off != 0)
 		buf[off - 1] = '\0';
 
-	if (paste_buffer != NULL)
-		xfree(paste_buffer);
-	paste_buffer = buf;
+	/* Add the buffer to the stack. */
+	limit = options_get_number(&c->session->options, "buffer-limit");
+	paste_add(&c->session->buffers, buf, limit);
+	xfree(buf);
 }
 
 void
@@ -505,7 +524,7 @@ window_copy_cursor_end_of_line(struct window *w)
 			    w, data->ox - (px - screen_last_x(s)));
 			data->cx = screen_last_x(s);
 		}
-	}
+ 	}
 
 	if (window_copy_update_selection(w))
 		window_copy_redraw_lines(w, data->cy, 1);
@@ -608,6 +627,171 @@ window_copy_cursor_down(struct window *w)
 		window_copy_cursor_end_of_line(w);
 }
 
+void
+window_copy_cursor_next_word(struct window *w)
+{
+	struct window_copy_mode_data	*data = w->modedata;
+	struct screen			*s = &data->screen;
+	u_int				 px, py, xx, skip;
+
+	px = data->ox + data->cx;
+	py = screen_y(&w->base, data->cy) - data->oy;
+	xx = window_copy_find_length(w, py);
+
+	skip = 1;
+	if (px < xx) {
+		/* If currently on a space, skip space. */
+		if (window_copy_is_space(w, px, py))
+			skip = 0;
+	} else
+		skip = 0;
+	for (;;) {
+		if (px >= xx) {
+			if (skip) {
+				px = xx;
+				break;
+			}
+
+			while (px >= xx) {
+				if (data->cy == screen_last_y(s)) {
+					if (data->oy == 0)
+						goto out;
+				}
+				
+				px = 0;
+				window_copy_cursor_down(w);
+				
+				py = screen_y(&w->base, data->cy) - data->oy;
+				xx = window_copy_find_length(w, py);
+			}
+		}
+		
+		if (skip) {
+			/* Currently skipping non-space (until space). */
+			if (window_copy_is_space(w, px, py))
+				break;
+		} else {
+			/* Currently skipping space (until non-space). */
+			if (!window_copy_is_space(w, px, py))
+				skip = 1;
+		}
+
+		px++;
+	}
+out:
+
+	/* On screen. */
+	if (px > data->ox && px <= data->ox + screen_last_x(s))
+		data->cx = px - data->ox;
+
+	/* Off right of screen. */
+	if (px > data->ox + screen_last_x(s)) {
+		/* Move cursor to last and scroll screen. */
+		window_copy_scroll_left(w,
+		    px - data->ox - screen_last_x(s));
+		data->cx = screen_last_x(s);
+	}
+
+	/* Off left of screen. */
+	if (px <= data->ox) {
+		if (px < screen_last_x(s)) {
+			/* Short enough to fit on screen. */
+			window_copy_scroll_right(w, data->ox);
+			data->cx = px;
+		} else {
+			/* Too long to fit on screen. */
+			window_copy_scroll_right(
+			    w, data->ox - (px - screen_last_x(s)));
+			data->cx = screen_last_x(s);
+		}
+ 	}
+ 
+	if (window_copy_update_selection(w))
+		window_copy_redraw_lines(w, data->cy, 1);
+	else
+		window_copy_update_cursor(w);
+}
+
+void
+window_copy_cursor_previous_word(struct window *w)
+{
+	struct window_copy_mode_data	*data = w->modedata;
+	struct screen			*s = &data->screen;
+	u_int				 ox, px, py, skip;
+
+	ox = px = data->ox + data->cx;
+	py = screen_y(&w->base, data->cy) - data->oy;
+
+	skip = 1;
+	if (px != 0) {
+		/* If currently on a space, skip space. */
+		if (window_copy_is_space(w, px - 1, py))
+			skip = 0;
+	}
+	for (;;) {
+		if (px == 0) {
+			if (ox != 0)
+				break;
+
+			while (px == 0) {
+				if (data->cy == 0 && (w->base.hsize == 0 ||
+				    data->oy >= w->base.hsize - 1))
+					goto out;
+				
+				window_copy_cursor_up(w);
+				
+				py = screen_y(&w->base, data->cy) - data->oy;
+				px = window_copy_find_length(w, py);
+			}
+			goto out;
+		}
+		
+		if (skip) {
+			/* Currently skipping non-space (until space). */
+			if (window_copy_is_space(w, px - 1, py))
+				skip = 0;
+		} else {
+			/* Currently skipping space (until non-space). */
+			if (!window_copy_is_space(w, px - 1, py))
+				break;
+		}
+		
+		px--;
+	}
+out:
+
+	/* On screen. */
+	if (px > data->ox && px <= data->ox + screen_last_x(s))
+		data->cx = px - data->ox;
+
+	/* Off right of screen. */
+	if (px > data->ox + screen_last_x(s)) {
+		/* Move cursor to last and scroll screen. */
+		window_copy_scroll_left(w,
+		    px - data->ox - screen_last_x(s));
+		data->cx = screen_last_x(s);
+	}
+
+	/* Off left of screen. */
+	if (px <= data->ox) {
+		if (px < screen_last_x(s)) {
+			/* Short enough to fit on screen. */
+			window_copy_scroll_right(w, data->ox);
+			data->cx = px;
+		} else {
+			/* Too long to fit on screen. */
+			window_copy_scroll_right(
+			    w, data->ox - (px - screen_last_x(s)));
+			data->cx = screen_last_x(s);
+		}
+ 	}
+ 
+	if (window_copy_update_selection(w))
+		window_copy_redraw_lines(w, data->cy, 1);
+	else
+		window_copy_update_cursor(w);
+}
+
 void
 window_copy_scroll_left(struct window *w, u_int nx)
 {
