@@ -1,4 +1,4 @@
-/* $Id: window-more.c,v 1.11 2008/06/03 21:42:37 nicm Exp $ */
+/* $Id: window-more.c,v 1.14 2008/06/22 16:56:47 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -25,7 +25,7 @@
 struct screen *window_more_init(struct window *);
 void	window_more_free(struct window *);
 void	window_more_resize(struct window *, u_int, u_int);
-void	window_more_key(struct window *, int);
+void	window_more_key(struct window *, struct client *, int);
 
 void	window_more_redraw_screen(struct window *);
 void	window_more_write_line(
@@ -94,7 +94,7 @@ window_more_init(struct window *w)
 	s = &data->screen;
 	screen_create(s, screen_size_x(&w->base), screen_size_y(&w->base), 0);
 	s->mode &= ~MODE_CURSOR;
-	
+
 	return (s);
 }
 
@@ -123,7 +123,7 @@ window_more_resize(struct window *w, u_int sx, u_int sy)
 }
 
 void
-window_more_key(struct window *w, int key)
+window_more_key(struct window *w, unused struct client *c, int key)
 {
 	struct window_more_mode_data	*data = w->modedata;
 	struct screen			*s = &data->screen;
@@ -172,7 +172,7 @@ window_more_write_line(struct window *w, struct screen_write_ctx *ctx, u_int py)
 
 	if (py == 0) {
 		screen_write_set_attributes(
-		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x70);
+		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x88);
 		screen_write_move_cursor(ctx, 0, 0);
 		size = screen_write_put_string_rjust(
 		    ctx, "[%u/%u]", data->top, ARRAY_LENGTH(&data->list));
