@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.41 2008/06/14 16:47:20 nicm Exp $ */
+/* $Id: window.c,v 1.47 2008/06/30 18:27:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -20,7 +20,6 @@
 #include <sys/ioctl.h>
 
 #include <fcntl.h>
-#include <paths.h>
 #include <signal.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -28,15 +27,21 @@
 #include <termios.h>
 #include <unistd.h>
 
+#ifndef NO_PATHS_H
+#include <paths.h>
+#endif
+
 #ifdef USE_LIBUTIL_H
 #include <libutil.h>
 #else
 #ifdef USE_PTY_H
 #include <pty.h>
 #else
+#ifndef NO_FORKPTY
 #include <util.h>
 #endif
 #endif
+#endif
 
 #include "tmux.h"
 
@@ -160,51 +165,26 @@ winlink_previous(unused struct winlinks *wwl, struct winlink *wl)
 }
 
 struct window *
-window_create(const char *name, 
-    const char *cmd, const char **env, u_int sx, u_int sy, u_int hlimit)
+window_create(const char *name,
+    const char *cmd, const char **envp, u_int sx, u_int sy, u_int hlimit)
 {
 	struct window	*w;
-	struct winsize	 ws;
-	int		 fd, mode;
 	char		*ptr, *copy;
-	const char     **entry;
-
-	memset(&ws, 0, sizeof ws);
-	ws.ws_col = sx;
-	ws.ws_row = sy;
-
- 	switch (forkpty(&fd, NULL, NULL, &ws)) {
-	case -1:
-		return (NULL);
-	case 0:
-		for (entry = env; *entry != NULL; entry++) {
-			if (putenv(*entry) != 0)
-				fatal("putenv failed");
-		}
-		sigreset();
-		log_debug("started child: cmd=%s; pid=%d", cmd, getpid());
-		log_close();
-		
-		execl(_PATH_BSHELL, "sh", "-c", cmd, (char *) NULL);
-		fatal("execl failed");
-	}
-
-	if ((mode = fcntl(fd, F_GETFL)) == -1)
-		fatal("fcntl failed");
-	if (fcntl(fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl failed");
-	if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
-		fatal("fcntl failed");
 
 	w = xmalloc(sizeof *w);
-	w->fd = fd;
+	w->cmd = NULL;
+
+	w->fd = -1;
 	w->in = buffer_create(BUFSIZ);
 	w->out = buffer_create(BUFSIZ);
+
 	w->mode = NULL;
 	w->flags = 0;
+
 	w->limitx = w->limity = UINT_MAX;
 	screen_create(&w->base, sx, sy, hlimit);
 	w->screen = &w->base;
+
 	input_init(w);
 
 	if (name == NULL) {
@@ -233,9 +213,58 @@ window_create(const char *name,
 	ARRAY_ADD(&windows, w);
 	w->references = 0;
 
+	if (window_spawn(w, cmd, envp) != 0) {
+		window_destroy(w);
+		return (NULL);
+	}
 	return (w);
 }
 
+int
+window_spawn(struct window *w, const char *cmd, const char **envp)
+{
+	struct winsize	 ws;
+	int		 mode;
+	const char     **envq;
+
+	if (w->fd != -1)
+		close(w->fd);
+	if (cmd != NULL) {
+		if (w->cmd != NULL)
+			xfree(w->cmd);
+		w->cmd = xstrdup(cmd);
+	}
+
+	memset(&ws, 0, sizeof ws);
+	ws.ws_col = screen_size_x(&w->base);
+	ws.ws_row = screen_size_y(&w->base);
+
+ 	switch (forkpty(&w->fd, NULL, NULL, &ws)) {
+	case -1:
+		return (1);
+	case 0:
+		for (envq = envp; *envq != NULL; envq++) {
+			if (putenv(*envq) != 0)
+				fatal("putenv failed");
+		}
+		sigreset();
+		log_debug("new child: cmd=%s; pid=%d", w->cmd, (int) getpid());
+		log_close();
+
+		execl(_PATH_BSHELL, "sh", "-c", w->cmd, (char *) NULL);
+		fatal("execl failed");
+	}
+
+	if ((mode = fcntl(w->fd, F_GETFL)) == -1)
+		fatal("fcntl failed");
+	if (fcntl(w->fd, F_SETFL, mode|O_NONBLOCK) == -1)
+		fatal("fcntl failed");
+	if (fcntl(w->fd, F_SETFD, FD_CLOEXEC) == -1)
+		fatal("fcntl failed");
+	
+	return (0);
+}
+
 void
 window_destroy(struct window *w)
 {
@@ -247,7 +276,8 @@ window_destroy(struct window *w)
 	}
 	ARRAY_REMOVE(&windows, i);
 
-	close(w->fd);
+	if (w->fd != -1)
+		close(w->fd);
 
 	input_free(w);
 
@@ -257,6 +287,8 @@ window_destroy(struct window *w)
 	buffer_destroy(w->in);
 	buffer_destroy(w->out);
 
+	if (w->cmd != NULL)
+		xfree(w->cmd);
 	xfree(w->name);
 	xfree(w);
 }
@@ -277,7 +309,7 @@ window_resize(struct window *w, u_int sx, u_int sy)
 	if (w->mode != NULL)
 		w->mode->resize(w, sx, sy);
 
-	if (ioctl(w->fd, TIOCSWINSZ, &ws) == -1)
+	if (w->fd != -1 && ioctl(w->fd, TIOCSWINSZ, &ws) == -1)
 		fatal("ioctl failed");
 	return (0);
 }
@@ -318,10 +350,10 @@ window_parse(struct window *w)
 }
 
 void
-window_key(struct window *w, int key)
+window_key(struct window *w, struct client *c, int key)
 {
 	if (w->mode != NULL)
-		w->mode->key(w, key);
+		w->mode->key(w, c, key);
 	else
 		input_key(w, key);
 }
